@@ -4,6 +4,6 @@ set -e
 
 if [ "$DISTRO" != "" ] ; then
   docker exec $(cat $HOME/cid)  ccache -s
-else
+elif [ "$TRAVIS_OS_NAME" = "osx" ]; then
   ccache -s
 fi
