@@ -19,19 +19,18 @@ test_ui_datepicker(void *data EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *e
    Eo *win, *bx;
 
    win = efl_add(EFL_UI_WIN_CLASS, efl_main_loop_get(),
-                 efl_ui_win_type_set(efl_added, EFL_UI_WIN_TYPE_BASIC),
-                 efl_text_set(efl_added, "Efl.Ui.Datepicker"),
+                                  efl_text_set(efl_added, "Efl.Ui.Datepicker"),
                  efl_ui_win_autodel_set(efl_added, EINA_TRUE));
 
    bx = efl_add(EFL_UI_BOX_CLASS, win,
                 efl_content_set(win, efl_added),
-                efl_ui_direction_set(efl_added, EFL_UI_DIR_DOWN));
+                efl_ui_layout_orientation_set(efl_added, EFL_UI_LAYOUT_ORIENTATION_VERTICAL));
 
    efl_add(EFL_UI_DATEPICKER_CLASS, bx,
            efl_ui_datepicker_date_set(efl_added, 1987, 9, 17),
-           efl_ui_datepicker_max_set(efl_added, 1990, 9, 17),
-           efl_ui_datepicker_min_set(efl_added, 1980, 9, 17),
-           efl_event_callback_add(efl_added, EFL_UI_DATEPICKER_EVENT_CHANGED,_date_changed_cb, NULL),
+           efl_ui_datepicker_date_max_set(efl_added, 1990, 9, 17),
+           efl_ui_datepicker_date_min_set(efl_added, 1980, 9, 17),
+           efl_event_callback_add(efl_added, EFL_UI_DATEPICKER_EVENT_DATE_CHANGED,_date_changed_cb, NULL),
            efl_pack(bx, efl_added));
 
    efl_gfx_entity_size_set(win, EINA_SIZE2D(150, 170));
