@@ -88,7 +88,6 @@ typedef struct _Ecore_Factorized_Idle Ecore_Factorized_Idle;
 typedef struct _Efl_Loop_Promise_Simple_Data Efl_Loop_Promise_Simple_Data;
 
 typedef struct _Efl_Loop_Timer_Data Efl_Loop_Timer_Data;
-typedef struct _Efl_Loop_Future_Scheduler Efl_Loop_Future_Scheduler;
 typedef struct _Efl_Loop_Data Efl_Loop_Data;
 
 typedef struct _Efl_Task_Data Efl_Task_Data;
@@ -104,20 +103,10 @@ struct _Message
    Eina_Bool delete_me;
 };
 
-struct _Efl_Loop_Future_Scheduler
-{
-   Eina_Future_Scheduler  eina_future_scheduler;
-   const Eo              *loop;
-
-   Eina_List             *future_entries;
-};
-
 struct _Efl_Loop_Data
 {
    double               loop_time;
 
-   Efl_Loop_Future_Scheduler future_scheduler;
-
    Efl_Loop_Message_Handler *future_message_handler;
 
    Efl_Loop_Timer      *poll_high;
@@ -314,8 +303,6 @@ void        *_ecore_factorized_idle_del(Ecore_Idler *idler);
 void    _ecore_factorized_idle_process(void *data, const Efl_Event *event);
 void    _ecore_factorized_idle_event_del(void *data, const Efl_Event *event);
 
-Eina_Future_Scheduler *_ecore_event_future_scheduler_get(void);
-
 Eina_Bool    _ecore_event_init(void);
 void         _ecore_event_shutdown(void);
 int          _ecore_event_exist(void);
@@ -369,7 +356,7 @@ _ecore_main_win32_handler_del(Eo *obj,
 void       _ecore_main_content_clear(Eo *obj, Efl_Loop_Data *pd);
 void       _ecore_main_shutdown(void);
 
-#if defined (_WIN32) || defined (__lv2ppu__) || defined (HAVE_EXOTIC)
+#if defined (_WIN32) || defined (__lv2ppu__)
 static inline void _ecore_signal_shutdown(void) { }
 
 static inline void _ecore_signal_init(void) { }
