@@ -25,8 +25,10 @@
 # include <sys/socket.h>
 # ifdef HAVE_PRCTL
 #  include <sys/prctl.h>
+# elif defined(HAVE_PROCCTL)
+#  include <sys/procctl.h>
 # endif
-# ifdef HAVE_SYS_WAIT_H
+#ifdef HAVE_SYS_WAIT_H
 #  include <sys/wait.h>
 # endif
 # ifndef HAVE_CLEARENV
@@ -67,6 +69,7 @@ struct _Efl_Exe_Data
    } fd;
 #endif
    Eina_Bool exit_called : 1;
+   Eina_Bool exit_signalled : 1;
    Eina_Bool run : 1;
 };
 
@@ -150,7 +153,13 @@ _exec(const char *cmd, Efl_Exe_Flags flags, Efl_Task_Flags task_flags)
 # ifdef HAVE_PRCTL
    if (task_flags & EFL_TASK_FLAGS_EXIT_WITH_PARENT)
      prctl(PR_SET_PDEATHSIG, SIGTERM);
-# endif
+#elif defined(HAVE_PROCCTL)
+   if (task_flags & EFL_TASK_FLAGS_EXIT_WITH_PARENT)
+     {
+        int sig = SIGTERM;
+        procctl(P_PID, 0, PROC_PDEATHSIG_CTL, &sig);
+     }
+#endif
 
    if (flags & EFL_EXE_FLAGS_GROUP_LEADER) setsid();
    if (use_sh) // We have to use a shell to run this.
@@ -261,10 +270,10 @@ _efl_exe_signal(Eo *obj EINA_UNUSED, Efl_Exe_Data *pd, Efl_Exe_Signal sig)
 
    switch (sig)
      {
-      case EFL_EXE_SIGNAL_INT:  s = SIGINT;  break;
-      case EFL_EXE_SIGNAL_QUIT: s = SIGQUIT; break;
-      case EFL_EXE_SIGNAL_TERM: s = SIGTERM; break;
-      case EFL_EXE_SIGNAL_KILL: s = SIGKILL; break;
+      case EFL_EXE_SIGNAL_INT:  s = SIGINT;  pd->exit_signalled = EINA_TRUE; break;
+      case EFL_EXE_SIGNAL_QUIT: s = SIGQUIT; pd->exit_signalled = EINA_TRUE; break;
+      case EFL_EXE_SIGNAL_TERM: s = SIGTERM; pd->exit_signalled = EINA_TRUE; break;
+      case EFL_EXE_SIGNAL_KILL: s = SIGKILL; pd->exit_signalled = EINA_TRUE; break;
       case EFL_EXE_SIGNAL_CONT: s = SIGCONT; break;
       case EFL_EXE_SIGNAL_STOP: s = SIGSTOP; break;
       case EFL_EXE_SIGNAL_HUP:  s = SIGHUP;  break;
@@ -540,6 +549,22 @@ _efl_exe_efl_task_run(Eo *obj, Efl_Exe_Data *pd)
        pd->env = NULL;
      }
 
+   // close all fd's other than the first 3 (0, 1, 2) and exited write fd
+   int except[2] = { 0, -1 };
+   except[0] = pd->fd.exited_write;
+   eina_file_close_from(3, except);
+#ifdef HAVE_PRCTL
+   if ((pd->flags & EFL_EXE_FLAGS_TERM_WITH_PARENT))
+     {
+        prctl(PR_SET_PDEATHSIG, SIGTERM);
+     }
+#elif defined(HAVE_PROCCTL)
+   if ((pd->flags & EFL_EXE_FLAGS_TERM_WITH_PARENT))
+     {
+        int sig = SIGTERM;
+        procctl(P_PID, 0, PROC_PDEATHSIG_CTL, &sig);
+     }
+#endif
    // actually execute!
    _exec(cmd, pd->flags, td->flags);
    // we couldn't exec... uh oh. HAAAAAAAALP!
@@ -557,6 +582,7 @@ _efl_exe_efl_task_end(Eo *obj EINA_UNUSED, Efl_Exe_Data *pd)
 #ifdef _WIN32
 #else
    if (pd->pid == -1) return;
+   pd->exit_signalled = EINA_TRUE;
    kill(pd->pid, SIGINT);
 #endif
 }
@@ -567,6 +593,16 @@ _efl_exe_exit_signal_get(const Eo *obj EINA_UNUSED, Efl_Exe_Data *pd)
    return pd->exit_signal;
 }
 
+EOLIAN static int
+_efl_exe_pid_get(const Eo *obj EINA_UNUSED, Efl_Exe_Data *pd)
+{
+#ifndef _WIN32
+   if (pd->pid != -1)
+     return pd->pid;
+#endif
+   return 0;
+}
+
 EOLIAN static Efl_Object *
 _efl_exe_efl_object_constructor(Eo *obj, Efl_Exe_Data *pd)
 {
@@ -589,7 +625,7 @@ _efl_exe_efl_object_destructor(Eo *obj, Efl_Exe_Data *pd)
 {
 #ifdef _WIN32
 #else
-   if (!pd->exit_called)
+   if ((!pd->exit_called) && (!pd->exit_signalled))
      ERR("Exe being destroyed while child has not exited yet.");
    if (pd->fd.exited_read >= 0)
      {
