@@ -137,10 +137,10 @@ _ecore_drm2_fb_destroy(Ecore_Drm2_Fb *fb)
 {
    EINA_SAFETY_ON_NULL_RETURN(fb);
 
-   if (!fb->dead) ERR("Destroying an fb that hasn't been discarded");
+   if (!fb->dead) WRN("Destroying an fb that hasn't been discarded");
 
    if (fb->scanout_count)
-     ERR("Destroyed fb on scanout %d times.", fb->scanout_count);
+     WRN("Destroyed fb on scanout %d times.", fb->scanout_count);
 
    if (fb->mmap) munmap(fb->mmap, fb->sizes[0]);
 
