@@ -33,7 +33,7 @@
 extern "C" {
 #endif
 
-#define ECORE_IMF_INPUT_HINT_AUTOFILL_MASK 0xff00
+#define ECORE_IMF_INPUT_HINT_AUTOFILL_MASK 0xff00 /* @deprecated since 1.24 */
 
 /**
  * @defgroup Ecore_IMF_Lib_Group Ecore_IMF - Ecore Input Method Library Functions
@@ -47,6 +47,8 @@ extern "C" {
  * @defgroup Ecore_IMF_Context_Group Ecore Input Method Context Functions
  * @ingroup Ecore_IMF_Lib_Group
  *
+ * @section ecore_imf_intro Introduction
+ *
  * Functions that operate on Ecore Input Method Context objects.
 
  * Ecore Input Method Context Function defines the interface for EFL input methods.
@@ -64,8 +66,44 @@ extern "C" {
  * ecore_imf_init() should be called to initialize and load immodule.@n
  * ecore_imf_shutdown() is used for shutdowning and unloading immodule.
  *
- * An example of usage of these functions can be found at:
+ * @section how-to-compose How to process key event for composition or prediction
+ *
+ * To input Chinese, Japanese, Korean and other complex languages, the editor widget (as known as entry) should be connected with input method framework.@n
+ * Each editor widget should have each input context to connect with input service framework.@n
+ * Key event is processed by input method engine. The result is notified to application through ECORE_IMF_CALLBACK_PREEDIT_CHANGED and ECORE_IMF_CALLBACK_COMMIT event.@n
+ * @n
+ * The following example demonstrates how to connect input method framework and handle preedit and commit string from input method framework.
  * @li @ref ecore_imf_example_c
+ *
+ * @section media-content How to receive media contents from input method editor
+ *
+ * Users sometimes wants to send images and other rich content with their input method editor (as known as virtual keyboard or soft keyboard).@n
+ * According to this requirement, the way to receive the media content URI such as images and other rich content as well as text have been provided since 1.20.@n
+ * @n
+ * The following code shows how to receive the media content URI.
+ *
+ * @code
+ * #include <glib.h>
+ *
+ * static void
+ * _imf_event_commit_content_cb(void *data, Ecore_IMF_Context *ctx, void *event_info)
+ * {
+ *    Ecore_IMF_Event_Commit_Content *commit_content = (Ecore_IMF_Event_Commit_Content *)event_info;
+ *    if (!commit_content) return;
+ *
+ *    // convert URI to filename
+ *    gchar *filepath = g_filename_from_uri(commit_content->content_uri, NULL, NULL);
+ *    printf("filepath : %s, description : %s, mime types : %s\n", filepath, commit_content->description, commit_content->mime_types);
+ *
+ *    // do something to use filepath
+ *
+ *    if (filepath)
+ *       g_free(filepath);
+ * }
+ *
+ * ecore_imf_context_event_callback_add(en->imf_context, ECORE_IMF_CALLBACK_COMMIT_CONTENT, _imf_event_commit_content_cb, data);
+ * @endcode
+ *
  */
 
 /**
@@ -375,17 +413,17 @@ typedef enum
    ECORE_IMF_INPUT_HINT_AUTO_COMPLETE       = 1 << 0,   /**< Suggest word auto completion @since 1.12 */
    ECORE_IMF_INPUT_HINT_SENSITIVE_DATA      = 1 << 1,   /**< Typed text should not be stored. @since 1.12 */
    ECORE_IMF_INPUT_HINT_MULTILINE           = 1 << 2,   /**< Multiline text @since 1.18 */
-   ECORE_IMF_INPUT_HINT_AUTOFILL_CREDIT_CARD_EXPIRATION_DATE   = 0x100, /**< Autofill hint for a credit card expiration date @since 1.21 */
-   ECORE_IMF_INPUT_HINT_AUTOFILL_CREDIT_CARD_EXPIRATION_DAY    = 0x200, /**< Autofill hint for a credit card expiration day @since 1.21 */
-   ECORE_IMF_INPUT_HINT_AUTOFILL_CREDIT_CARD_EXPIRATION_MONTH  = 0x300, /**< Autofill hint for a credit card expiration month @since 1.21 */
-   ECORE_IMF_INPUT_HINT_AUTOFILL_CREDIT_CARD_EXPIRATION_YEAR   = 0x400, /**< Autofill hint for a credit card expiration year @since 1.21 */
-   ECORE_IMF_INPUT_HINT_AUTOFILL_CREDIT_CARD_NUMBER            = 0x500, /**< Autofill hint for a credit card number @since 1.21 */
-   ECORE_IMF_INPUT_HINT_AUTOFILL_EMAIL_ADDRESS                 = 0x600, /**< Autofill hint for an email address @since 1.21 */
-   ECORE_IMF_INPUT_HINT_AUTOFILL_NAME                          = 0x700, /**< Autofill hint for a user's real name @since 1.21 */
-   ECORE_IMF_INPUT_HINT_AUTOFILL_PHONE                         = 0x800, /**< Autofill hint for a phone number @since 1.21 */
-   ECORE_IMF_INPUT_HINT_AUTOFILL_POSTAL_ADDRESS                = 0x900, /**< Autofill hint for a postal address @since 1.21 */
-   ECORE_IMF_INPUT_HINT_AUTOFILL_POSTAL_CODE                   = 0xA00, /**< Autofill hint for a postal code @since 1.21 */
-   ECORE_IMF_INPUT_HINT_AUTOFILL_ID                            = 0xB00  /**< Autofill hint for a user's ID @since 1.21 */
+   ECORE_IMF_INPUT_HINT_AUTOFILL_CREDIT_CARD_EXPIRATION_DATE   = 0x100, /**< Autofill hint for a credit card expiration date @deprecated since 1.24 */
+   ECORE_IMF_INPUT_HINT_AUTOFILL_CREDIT_CARD_EXPIRATION_DAY    = 0x200, /**< Autofill hint for a credit card expiration day @deprecated since 1.24 */
+   ECORE_IMF_INPUT_HINT_AUTOFILL_CREDIT_CARD_EXPIRATION_MONTH  = 0x300, /**< Autofill hint for a credit card expiration month @deprecated since 1.24 */
+   ECORE_IMF_INPUT_HINT_AUTOFILL_CREDIT_CARD_EXPIRATION_YEAR   = 0x400, /**< Autofill hint for a credit card expiration year @deprecated since 1.24 */
+   ECORE_IMF_INPUT_HINT_AUTOFILL_CREDIT_CARD_NUMBER            = 0x500, /**< Autofill hint for a credit card number @deprecated since 1.24 */
+   ECORE_IMF_INPUT_HINT_AUTOFILL_EMAIL_ADDRESS                 = 0x600, /**< Autofill hint for an email address @deprecated since 1.24 */
+   ECORE_IMF_INPUT_HINT_AUTOFILL_NAME                          = 0x700, /**< Autofill hint for a user's real name @deprecated since 1.24 */
+   ECORE_IMF_INPUT_HINT_AUTOFILL_PHONE                         = 0x800, /**< Autofill hint for a phone number @deprecated since 1.24 */
+   ECORE_IMF_INPUT_HINT_AUTOFILL_POSTAL_ADDRESS                = 0x900, /**< Autofill hint for a postal address @deprecated since 1.24 */
+   ECORE_IMF_INPUT_HINT_AUTOFILL_POSTAL_CODE                   = 0xA00, /**< Autofill hint for a postal code @deprecated since 1.24 */
+   ECORE_IMF_INPUT_HINT_AUTOFILL_ID                            = 0xB00  /**< Autofill hint for a user's ID @deprecated since 1.24 */
 } Ecore_IMF_Input_Hints;
 
 /**
@@ -674,6 +712,7 @@ struct _Ecore_IMF_Event_Key_Down
    const char                   *dev_name;  /**< The device name of the key pressed @since 1.14 */
    Ecore_IMF_Device_Class        dev_class; /**< The device class of the key pressed @since 1.14 */
    Ecore_IMF_Device_Subclass     dev_subclass; /**< The device subclass of the key pressed @since 1.14 */
+   unsigned int                  keycode;   /**< Key scan code numeric value @since 1.22 */
 };
 
 /**
@@ -692,6 +731,7 @@ struct _Ecore_IMF_Event_Key_Up
    const char                   *dev_name;  /**< The device name of the key released @since 1.14 */
    Ecore_IMF_Device_Class        dev_class; /**< The device class of the key released @since 1.14 */
    Ecore_IMF_Device_Subclass     dev_subclass; /**< The device subclass of the key released @since 1.14 */
+   unsigned int                  keycode;   /**< Key scan code numeric value @since 1.22 */
 };
 
 /**
@@ -995,16 +1035,20 @@ EAPI void                         *ecore_imf_context_client_canvas_get(Ecore_IMF
  * @brief Asks the Input Method Context to show itself.
  *
  * @param ctx An #Ecore_IMF_Context.
+ *
+ * @deprecated use ecore_imf_context_input_panel_show() instead.
  */
-EAPI void                          ecore_imf_context_show(Ecore_IMF_Context *ctx);
+EINA_DEPRECATED EAPI void          ecore_imf_context_show(Ecore_IMF_Context *ctx);
 
 /**
  * @ingroup Ecore_IMF_Context_Group
  * @brief Asks the Input Method Context to hide itself.
  *
  * @param ctx An #Ecore_IMF_Context.
+ *
+ * @deprecated use ecore_imf_context_input_panel_hide() instead.
  */
-EAPI void                          ecore_imf_context_hide(Ecore_IMF_Context *ctx);
+EINA_DEPRECATED EAPI void          ecore_imf_context_hide(Ecore_IMF_Context *ctx);
 
 /**
  * @ingroup Ecore_IMF_Context_Group
@@ -1386,8 +1430,10 @@ EAPI Eina_Bool                     ecore_imf_context_selection_get(Ecore_IMF_Con
  * ecore_imf_context_event_callback_call() can be used as synchronous method.
  *
  * @param ctx An #Ecore_IMF_Context.
+ *
+ * @deprecated use ecore_imf_context_event_callback_call() instead.
  */
-EAPI void                          ecore_imf_context_preedit_start_event_add(Ecore_IMF_Context *ctx);
+EINA_DEPRECATED EAPI void          ecore_imf_context_preedit_start_event_add(Ecore_IMF_Context *ctx);
 
 /**
  * @ingroup Ecore_IMF_Context_Module_Group
@@ -1398,8 +1444,10 @@ EAPI void                          ecore_imf_context_preedit_start_event_add(Eco
  * ecore_imf_context_event_callback_call() can be used as synchronous method.
  *
  * @param ctx An #Ecore_IMF_Context.
+ *
+ * @deprecated use ecore_imf_context_event_callback_call() instead.
  */
-EAPI void                          ecore_imf_context_preedit_end_event_add(Ecore_IMF_Context *ctx);
+EINA_DEPRECATED EAPI void          ecore_imf_context_preedit_end_event_add(Ecore_IMF_Context *ctx);
 
 /**
  * @ingroup Ecore_IMF_Context_Module_Group
@@ -1409,8 +1457,10 @@ EAPI void                          ecore_imf_context_preedit_end_event_add(Ecore
  * ecore_imf_context_event_callback_call() can be used as synchronous method.
  *
  * @param ctx An #Ecore_IMF_Context.
+ *
+ * @deprecated use ecore_imf_context_event_callback_call() instead.
  */
-EAPI void                          ecore_imf_context_preedit_changed_event_add(Ecore_IMF_Context *ctx);
+EINA_DEPRECATED EAPI void          ecore_imf_context_preedit_changed_event_add(Ecore_IMF_Context *ctx);
 
 /**
  * @ingroup Ecore_IMF_Context_Module_Group
@@ -1421,8 +1471,10 @@ EAPI void                          ecore_imf_context_preedit_changed_event_add(E
  *
  * @param ctx An #Ecore_IMF_Context.
  * @param str The committed string.
+ *
+ * @deprecated use ecore_imf_context_event_callback_call() instead.
  */
-EAPI void                          ecore_imf_context_commit_event_add(Ecore_IMF_Context *ctx, const char *str);
+EINA_DEPRECATED EAPI void          ecore_imf_context_commit_event_add(Ecore_IMF_Context *ctx, const char *str);
 
 /**
  * @ingroup Ecore_IMF_Context_Module_Group
@@ -1438,8 +1490,10 @@ EAPI void                          ecore_imf_context_commit_event_add(Ecore_IMF_
  * @param ctx An #Ecore_IMF_Context.
  * @param offset The start offset of surrounding to be deleted.
  * @param n_chars The number of characters to be deleted.
+ *
+ * @deprecated use ecore_imf_context_event_callback_call() instead.
  */
-EAPI void                          ecore_imf_context_delete_surrounding_event_add(Ecore_IMF_Context *ctx, int offset, int n_chars);
+EINA_DEPRECATED EAPI void          ecore_imf_context_delete_surrounding_event_add(Ecore_IMF_Context *ctx, int offset, int n_chars);
 
 /**
  * @ingroup Ecore_IMF_Context_Group
@@ -1451,8 +1505,10 @@ EAPI void                          ecore_imf_context_delete_surrounding_event_ad
  *
  * The event type @p type to trigger the function may be one of
  * #ECORE_IMF_CALLBACK_PREEDIT_START, #ECORE_IMF_CALLBACK_PREEDIT_END,
- * #ECORE_IMF_CALLBACK_PREEDIT_CHANGED, #ECORE_IMF_CALLBACK_COMMIT and
- * #ECORE_IMF_CALLBACK_DELETE_SURROUNDING.
+ * #ECORE_IMF_CALLBACK_PREEDIT_CHANGED, #ECORE_IMF_CALLBACK_COMMIT,
+ * #ECORE_IMF_CALLBACK_DELETE_SURROUNDING, #ECORE_IMF_CALLBACK_SELECTION_SET,
+ * #ECORE_IMF_CALLBACK_PRIVATE_COMMAND_SEND, #ECORE_IMF_CALLBACK_COMMIT_CONTENT,
+ * #ECORE_IMF_CALLBACK_TRANSACTION_START, and #ECORE_IMF_CALLBACK_TRANSACTION_END.
  *
  * @param ctx Ecore_IMF_Context to attach a callback to.
  * @param type The type of event that will trigger the callback
@@ -1463,6 +1519,9 @@ EAPI void                          ecore_imf_context_delete_surrounding_event_ad
  *
  * Example
  * @code
+ * #include <glib.h>
+ *
+ * // example for handling commit event from input framework
  * static void
  * _imf_event_commit_cb(void *data, Ecore_IMF_Context *ctx, void *event_info)
  * {
@@ -1471,6 +1530,28 @@ EAPI void                          ecore_imf_context_delete_surrounding_event_ad
  * }
  *
  * ecore_imf_context_event_callback_add(en->imf_context, ECORE_IMF_CALLBACK_COMMIT, _imf_event_commit_cb, data);
+ *
+ * // example for receiving media content URI from input framework
+ * @code
+ * #include <glib.h>
+ *
+ * static void
+ * _imf_event_commit_content_cb(void *data, Ecore_IMF_Context *ctx, void *event_info)
+ * {
+ *    Ecore_IMF_Event_Commit_Content *commit_content = (Ecore_IMF_Event_Commit_Content *)event;
+ *    if (!commit_content) return;
+ *
+ *    // convert URI to filename
+ *    gchar *filepath = g_filename_from_uri(commit_content->content_uri, NULL, NULL);
+ *    printf("filepath : %s, description : %s, mime types : %s\n", filepath, commit_content->description, commit_content->mime_types);
+ *
+ *    // do something to use filepath
+ *
+ *    if (filepath)
+ *       g_free(filepath);
+ * }
+ *
+ * ecore_imf_context_event_callback_add(en->imf_context, ECORE_IMF_CALLBACK_COMMIT_CONTENT, _imf_event_commit_content_cb, data);
  * @endcode
  */
 EAPI void                          ecore_imf_context_event_callback_add(Ecore_IMF_Context *ctx, Ecore_IMF_Callback_Type type, Ecore_IMF_Event_Cb func, const void *data);
@@ -1586,7 +1667,7 @@ EAPI Ecore_IMF_Input_Hints         ecore_imf_context_input_hint_get(Ecore_IMF_Co
  * @param ctx An #Ecore_IMF_Context.
  * @since 1.1.0
  */
-EAPI void                          ecore_imf_context_control_panel_show(Ecore_IMF_Context *ctx);
+EINA_DEPRECATED EAPI void          ecore_imf_context_control_panel_show(Ecore_IMF_Context *ctx);
 
 /**
  * @ingroup Ecore_IMF_Context_Group
@@ -1595,7 +1676,7 @@ EAPI void                          ecore_imf_context_control_panel_show(Ecore_IM
  * @param ctx An #Ecore_IMF_Context.
  * @since 1.1.0
  */
-EAPI void                          ecore_imf_context_control_panel_hide(Ecore_IMF_Context *ctx);
+EINA_DEPRECATED EAPI void          ecore_imf_context_control_panel_hide(Ecore_IMF_Context *ctx);
 
 /**
  * @ingroup Ecore_IMF_Context_Group
@@ -1978,7 +2059,7 @@ EAPI void                         ecore_imf_context_mime_type_accept_set(Ecore_I
 /**
  * @ingroup Ecore_IMF_Context_Group
  * @brief Sets the x,y coordinates of the input panel.
- * This API can be used in floating mode.
+ * @remarks This API can be used in floating mode.
  *
  * @since 1.21.0
  *
@@ -1996,7 +2077,7 @@ EAPI void                         ecore_imf_context_input_panel_position_set(Eco
  *
  * @param[in] ctx An #Ecore_IMF_Context
  * @param key The key of the prediction hint
- * @param data The data to replace
+ * @param value The data to replace
  * @return @c EINA_TRUE on success, @c EINA_FALSE otherwise
  *
  * This function modifies the data of @p key with @p data in the hash associated @p
