@@ -1881,14 +1881,6 @@ _edje_file_del(Edje *ed)
              if (rp->part->entry_mode > EDJE_ENTRY_EDIT_MODE_NONE)
                _edje_entry_real_part_shutdown(ed, rp);
 
-             if (rp->object)
-               {
-                  _edje_callbacks_focus_del(rp->object, ed);
-                  _edje_callbacks_del(rp->object, ed);
-                  evas_object_del(rp->object);
-                  rp->object = NULL;
-               }
-
              if (rp->custom)
                {
                   // xxx: lua2
@@ -1956,7 +1948,10 @@ _edje_file_del(Edje *ed)
                       (rp->typedata.vector))
                {
                   if (rp->typedata.vector->anim)
-                    efl_del(rp->typedata.vector->anim);
+                    {
+                       efl_canvas_object_animation_stop(rp->object);
+                       efl_del(rp->typedata.vector->anim);
+                    }
                   if (rp->typedata.vector->lottie_virtual_file)
                     eina_file_close(rp->typedata.vector->lottie_virtual_file);
                   if (rp->typedata.vector->lottie_data)
@@ -1966,6 +1961,14 @@ _edje_file_del(Edje *ed)
                   rp->typedata.vector = NULL;
                }
 
+             if (rp->object)
+               {
+                  _edje_callbacks_focus_del(rp->object, ed);
+                  _edje_callbacks_del(rp->object, ed);
+                  evas_object_del(rp->object);
+                  rp->object = NULL;
+               }
+
              /* Cleanup optional part. */
              free(rp->drag);
              rp->drag = NULL;
@@ -2118,6 +2121,9 @@ _edje_file_free(Edje_File *edf)
           {
              for (i = 0; i < edf->image_dir->entries_count; ++i)
                eina_stringshare_del(edf->image_dir->entries[i].entry);
+
+             for (i = 0; i < edf->image_dir->vectors_count; ++i)
+               eina_stringshare_del(edf->image_dir->vectors[i].entry);
           }
 
         /* Sets have been added after edje received eet dictionary support */
@@ -2131,6 +2137,7 @@ _edje_file_free(Edje_File *edf)
 
         free(edf->image_dir->entries);
         free(edf->image_dir->sets);
+        free(edf->image_dir->vectors);
         free(edf->image_dir);
      }
    if (edf->sound_dir)
@@ -2169,6 +2176,22 @@ _edje_file_free(Edje_File *edf)
         free(edf->vibration_dir);
      }
 
+   if (edf->filter_dir)
+     {
+        int i;
+
+        if (edf->free_strings)
+          {
+             for (i = 0; i < edf->filter_dir->filters_count; ++i)
+               {
+                  eina_stringshare_del(edf->filter_dir->filters[i].name);
+                  eina_stringshare_del(edf->filter_dir->filters[i].script);
+               }
+          }
+        free(edf->filter_dir->filters);
+        free(edf->filter_dir);
+     } 
+
    if (edf->mo_dir)
      {
         unsigned int i;
@@ -2349,6 +2372,8 @@ _edje_collection_free(Edje_File *edf, Edje_Part_Collection *ec, Edje_Part_Collec
    if (ec->script) embryo_program_free(ec->script);
    _edje_lua2_script_unload(ec);
 
+   if (ec->limits.parts) free(ec->limits.parts);
+
    eina_hash_free(ec->alias);
    eina_hash_free(ec->aliased);
 
