@@ -41,10 +41,6 @@
 #include "eina_file_common.h"
 #include "eina_xattr.h"
 
-#ifdef HAVE_ESCAPE
-# include <Escape.h>
-#endif
-
 #ifndef O_BINARY
 # define O_BINARY 0
 #endif
@@ -1154,7 +1150,11 @@ eina_file_shutdown(void)
 EAPI Eina_Bool
 eina_file_close_on_exec(int fd, Eina_Bool on)
 {
-#ifdef HAVE_FCNTL
+#ifdef _WIN32
+   return EINA_TRUE;
+   (void)fd;
+   (void)on;
+#elif HAVE_FCNTL
    int flags;
 
    flags = fcntl(fd, F_GETFD);
