@@ -38,7 +38,7 @@ struct _Efl_Ui_Collection_Item_Lookup
 {
    EINA_RBTREE;
 
-   uint64_t index;
+   unsigned int index;
    Efl_Ui_Collection_Item item;
 };
 
@@ -46,7 +46,7 @@ struct _Efl_Ui_Collection_Viewport
 {
    Efl_Ui_Collection_Item *items;
 
-   uint64_t offset;
+   unsigned int offset;
    uint16_t count;
 };
 
@@ -54,11 +54,10 @@ struct _Efl_Ui_Collection_Request
 {
    Eina_Future *f;
 
-   uint64_t offset;
-   uint64_t length;
+   unsigned int offset;
+   unsigned int length;
 
-   Eina_Bool model_requested : 1;
-   Eina_Bool model_fetched : 1;
+   Eina_Bool need_size : 1;
    Eina_Bool need_entity : 1;
    Eina_Bool entity_requested : 1;
 };
@@ -80,8 +79,13 @@ struct _Efl_Ui_Collection_View_Data
 
    Eina_List *requests; // Array of Efl_Ui_Collection_Request in progress
 
-   uint64_t start_id;
-   uint64_t end_id;
+   struct {
+      Efl_Gfx_Entity *last; // The last item of the collection, so focus can start by the end if necessary.
+      Efl_Gfx_Entity *previously; // The previously selected item in the collection, so focus can come back to it.
+   } focus;
+
+   unsigned int start_id;
+   unsigned int end_id;
 
    Eina_Size2D content_min_size;
 
@@ -117,7 +121,7 @@ _cache_tree_lookup(const Eina_Rbtree *node, const void *key,
                    int length EINA_UNUSED, void *data EINA_UNUSED)
 {
    const Efl_Ui_Collection_Item_Lookup *n = (Efl_Ui_Collection_Item_Lookup *)node;
-   const uint64_t *index = key;
+   const unsigned int *index = key;
 
    if (n->index > *index)
      return 1;
@@ -294,6 +298,9 @@ _all_cleanup(Efl_Ui_Collection_View *obj, Efl_Ui_Collection_View_Data *pd)
      }
 #endif
 
+   efl_replace(&pd->focus.previously, NULL);
+   efl_replace(&pd->focus.last, NULL);
+
    EINA_LIST_FOREACH_SAFE(pd->requests, l, ll, request)
      eina_future_cancel(request->f);
 }
@@ -345,13 +352,14 @@ _size_to_model(Efl_Model *model, Eina_Size2D state)
 
 static Eina_List *
 _request_add(Eina_List *requests, Efl_Ui_Collection_Request **request,
-             uint64_t index, Eina_Bool need_entity)
+             unsigned int index, Eina_Bool need_entity)
 {
    if (!(*request)) goto create;
 
    if ((*request)->offset + (*request)->length == index)
      {
         if (need_entity) (*request)->need_entity = EINA_TRUE;
+        if (!need_entity) (*request)->need_size = EINA_TRUE;
         (*request)->length += 1;
         return requests;
      }
@@ -364,8 +372,8 @@ _request_add(Eina_List *requests, Efl_Ui_Collection_Request **request,
    (*request)->offset = index;
    (*request)->length = 1;
    // At this point, we rely on the model caching ability to avoid recreating model
-   (*request)->model_requested = EINA_TRUE;
    (*request)->need_entity = !!need_entity;
+   (*request)->need_size = EINA_TRUE;
 
    return requests;
 }
@@ -379,7 +387,6 @@ _model_fetched_cb(Eo *obj, void *data, const Eina_Value v)
    unsigned int i, len;
    Eina_Bool request_entity = EINA_FALSE;
 
-   request->model_fetched = EINA_TRUE;
    EINA_VALUE_ARRAY_FOREACH(&v, len, i, child)
      {
         Efl_Ui_Collection_Item_Lookup *insert;
@@ -394,7 +401,7 @@ _model_fetched_cb(Eo *obj, void *data, const Eina_Value v)
              if ((pd->viewport[v]->offset <= request->offset + i) &&
                  (request->offset + i < pd->viewport[v]->offset + pd->viewport[v]->count))
                {
-                  uint64_t index = request->offset + i - pd->viewport[v]->offset;
+                  unsigned int index = request->offset + i - pd->viewport[v]->offset;
 
                   efl_replace(&pd->viewport[v]->items[index].model, child);
                   child = NULL;
@@ -406,7 +413,7 @@ _model_fetched_cb(Eo *obj, void *data, const Eina_Value v)
         // When requesting a model, it should not be in the cache prior to the request
         if (!child) continue;
 
-        uint64_t search_index = request->offset + i;
+        unsigned int search_index = request->offset + i;
 
         insert = (void*) eina_rbtree_inline_lookup(pd->cache, &search_index,
                                                    sizeof (search_index), _cache_tree_lookup,
@@ -419,7 +426,7 @@ _model_fetched_cb(Eo *obj, void *data, const Eina_Value v)
                   efl_replace(&insert->item.model, child);
                }
              else
-               ERR("Inserting a model that was already fetched, dropping new model %" PRIu64, search_index);
+               ERR("Inserting a model that was already fetched, dropping new model %u", search_index);
           }
         else
           {
@@ -435,11 +442,17 @@ _model_fetched_cb(Eo *obj, void *data, const Eina_Value v)
      }
 
    if (request_entity)
-     request->need_entity = EINA_TRUE;
+     {
+        request->need_entity = EINA_TRUE;
 
-   if (request->need_entity &&
-       !request->entity_requested)
-     _entity_request(obj, request);
+        if (!request->entity_requested)
+          _entity_request(obj, request);
+     }
+   else if (request->need_size)
+     {
+        efl_ui_position_manager_entity_item_size_changed(pd->manager, request->offset,
+                                                         request->offset + len);
+     }
 
    return v;
 }
@@ -480,6 +493,33 @@ _entity_fetch_cb(Eo *obj, void *data EINA_UNUSED, const Eina_Value v)
    return eina_future_as_value(r);
 }
 
+static inline unsigned int
+_lookup_entity_index(Efl_Gfx_Entity *entity, Efl_Model **model)
+{
+   Efl_Model *fetch;
+
+   fetch = efl_ui_view_model_get(entity);
+   if (model) *model = fetch;
+   return efl_composite_model_index_get(fetch);
+}
+
+static void
+_last_entity_update(Efl_Ui_Collection_View_Data *pd, Efl_Gfx_Entity *entity)
+{
+   Efl_Model *new_model, *old_model;
+   unsigned int new_index, old_index;
+
+   if (!pd->focus.last) goto replace;
+
+   new_index = _lookup_entity_index(entity, &new_model);
+   old_index = _lookup_entity_index(pd->focus.last, &old_model);
+
+   if (new_index <= old_index) return;
+
+ replace:
+   efl_replace(&pd->focus.last, entity);
+}
+
 static inline Eina_Bool
 _entity_propagate(Efl_Model *model, Efl_Gfx_Entity *entity)
 {
@@ -513,13 +553,17 @@ _entity_fetched_cb(Eo *obj, void *data, const Eina_Value v)
    Efl_Ui_Collection_Request *request = data;
    Efl_Gfx_Entity *child;
    unsigned int i, len;
-   uint64_t updated_size_start_id = 0, updated_entity_start_id = 0;
+   unsigned int updated_size_start_id = 0, updated_entity_start_id = 0;
    Eina_Bool updated_size = EINA_FALSE, updated_entity = EINA_FALSE;
+   Evas *e;
+
+   e = evas_object_evas_get(obj);
+   evas_event_freeze(e);
 
    EINA_VALUE_ARRAY_FOREACH(&v, len, i, child)
      {
         Efl_Ui_Collection_Item_Lookup *lookup;
-        uint64_t search_index;
+        unsigned int search_index;
         //unsigned int v;
 
         efl_key_data_set(child, COLLECTION_VIEW_MANAGED, COLLECTION_VIEW_MANAGED_YES);
@@ -537,7 +581,7 @@ _entity_fetched_cb(Eo *obj, void *data, const Eina_Value v)
              if ((pd->viewport[v]->offset <= request->offset + i) &&
                  (request->offset + i < pd->viewport[v]->offset + pd->viewport[v]->count))
                {
-                  uint64_t index = request->offset + i - pd->viewport[v]->offset;
+                  unsigned int index = request->offset + i - pd->viewport[v]->offset;
 
                   if (pd->viewport[v]->items[index].entity)
                     {
@@ -588,7 +632,7 @@ _entity_fetched_cb(Eo *obj, void *data, const Eina_Value v)
           }
         if (lookup->item.entity)
           {
-             ERR("Entity already existing for id %" PRIu64, search_index);
+             ERR("Entity already existing for id %u", search_index);
              _entity_cleanup(obj, pd->factory, &lookup->item, NULL);
           }
 
@@ -622,9 +666,15 @@ _entity_fetched_cb(Eo *obj, void *data, const Eina_Value v)
           }
      }
 
+   evas_event_thaw(e);
+   evas_event_thaw_eval(e);
+
+   // Check if the last child is also the list item in the list
+   _last_entity_update(pd, child);
+
    // Currently position manager will flush its entire size cache on update, so only do
    // it when necessary to improve performance.
-   if (updated_size)
+   if (updated_size || request->need_size)
      {
         efl_ui_position_manager_entity_item_size_changed(pd->manager,
                                                          updated_size_start_id,
@@ -655,15 +705,63 @@ _entity_free_cb(Eo *o, void *data, const Eina_Future *dead_future EINA_UNUSED)
    free(request);
 }
 
+static Eina_Bool
+_focus_lookup(Efl_Ui_Collection_View_Data *pd, unsigned int search_index,
+              Efl_Gfx_Entity **entity, Efl_Model **model)
+{
+   unsigned int idx;
+
+   if (entity) *entity = pd->focus.last;
+   if (pd->focus.last)
+     {
+        idx = _lookup_entity_index(pd->focus.last, model);
+        if (idx == search_index) return EINA_TRUE;
+     }
+   if (entity) *entity = pd->focus.previously;
+   if (pd->focus.previously)
+     {
+        idx = _lookup_entity_index(pd->focus.previously, model);
+        if (idx == search_index) return EINA_TRUE;
+     }
+
+   if (entity) *entity = NULL;
+   if (model) *model = NULL;
+   return EINA_FALSE;
+}
+
+static Efl_Ui_Collection_Item_Lookup *
+_build_from_focus(Efl_Ui_Collection_View_Data *pd, unsigned int search_index,
+                  Efl_Model **model)
+{
+   Efl_Ui_Collection_Item_Lookup *insert;
+   Efl_Gfx_Entity *entity = NULL;
+
+   // Not found in the cache lookup, but just maybe
+   if (!_focus_lookup(pd, search_index, &entity, model)) return NULL;
+
+   // Lucky us, let's add it to the cache
+   insert = calloc(1, sizeof (Efl_Ui_Collection_Item_Lookup));
+   if (!insert) return NULL;
+
+   insert->index = search_index;
+   insert->item.model = efl_ref(*model);
+   insert->item.entity = efl_ref(entity);
+
+   pd->cache = eina_rbtree_inline_insert(pd->cache, EINA_RBTREE_GET(insert),
+                                         _cache_tree_cmp, NULL);
+
+   return insert;
+}
+
 static Eina_List *
 _cache_size_fetch(Eina_List *requests, Efl_Ui_Collection_Request **request,
                   Efl_Ui_Collection_View_Data *pd,
-                  uint64_t search_index,
+                  unsigned int search_index,
                   Efl_Ui_Position_Manager_Size_Batch_Entity *target,
                   Eina_Size2D item_base)
 {
    Efl_Ui_Collection_Item_Lookup *lookup;
-   Efl_Model *model;
+   Efl_Model *model = NULL;
    Eina_Size2D item_size = item_base;
 
    if (!pd->cache) goto not_found;
@@ -671,10 +769,16 @@ _cache_size_fetch(Eina_List *requests, Efl_Ui_Collection_Request **request,
    lookup = (void*) eina_rbtree_inline_lookup(pd->cache, &search_index,
                                               sizeof (search_index), _cache_tree_lookup,
                                               NULL);
-   if (!lookup) goto not_found;
-
    // In the cache we should always have model, so no need to check for it
-   model = lookup->item.model;
+   if (lookup)
+     {
+        model = lookup->item.model;
+     }
+   else
+     {
+        lookup = _build_from_focus(pd, search_index, &model);
+        if (!lookup) goto not_found;
+     }
 
    // If we do not know the size
    if (!ITEM_SIZE_FROM_MODEL(model, item_size))
@@ -715,16 +819,18 @@ _cache_size_fetch(Eina_List *requests, Efl_Ui_Collection_Request **request,
 static Eina_List *
 _cache_entity_fetch(Eina_List *requests, Efl_Ui_Collection_Request **request,
                     Efl_Ui_Collection_View_Data *pd,
-                    uint64_t search_index,
+                    unsigned int search_index,
                     Efl_Ui_Position_Manager_Object_Batch_Entity *target)
 {
    Efl_Ui_Collection_Item_Lookup *lookup;
+   Efl_Model *model = NULL;
 
    if (!pd->cache) goto not_found;
 
    lookup = (void*) eina_rbtree_inline_lookup(pd->cache, &search_index,
                                               sizeof (search_index), _cache_tree_lookup,
                                               NULL);
+   if (!lookup) lookup = _build_from_focus(pd, search_index, &model);
    if (!lookup) goto not_found;
    if (!lookup->item.entity) goto not_found;
 
@@ -748,7 +854,6 @@ static Eina_Bool
 _entity_request(Efl_Ui_Collection_View *obj, Efl_Ui_Collection_Request *request)
 {
    if (request->entity_requested) return EINA_TRUE;
-   if (request->model_fetched) return EINA_FALSE;
    request->f = efl_future_then(obj, request->f,
                                 .success_type = EINA_VALUE_TYPE_ARRAY,
                                 .success = _entity_fetch_cb);
@@ -768,6 +873,7 @@ _entity_inflight_request(Efl_Ui_Collection_View *obj,
                          Efl_Ui_Collection_Request *request,
                          Efl_Ui_Collection_Request *inflight)
 {
+   inflight->need_size |= request->need_size;
    if (request->need_entity == EINA_FALSE) return EINA_TRUE;
 
    return _entity_request(obj, inflight);
@@ -790,10 +896,10 @@ _batch_request_flush(Eina_List *requests,
 
         EINA_LIST_FOREACH(pd->requests, l, inflight)
           {
-             uint64_t istart = inflight->offset;
-             uint64_t iend = inflight->offset + inflight->length;
-             uint64_t rstart = request->offset;
-             uint64_t rend = request->offset + request->length;
+             unsigned int istart = inflight->offset;
+             unsigned int iend = inflight->offset + inflight->length;
+             unsigned int rstart = request->offset;
+             unsigned int rend = request->offset + request->length;
 
              // Way before
              if (rend < istart) continue;
@@ -825,8 +931,8 @@ _batch_request_flush(Eina_List *requests,
 
                   rn->offset = iend;
                   rn->length = rend - iend;
-                  rn->model_requested = request->model_requested;
                   rn->need_entity = request->need_entity;
+                  rn->need_size = request->need_size;
 
                   requests = eina_list_append(requests, rn);
 
@@ -877,12 +983,6 @@ _batch_request_flush(Eina_List *requests,
 
         // We now have a request, time to trigger a fetch
         // We assume here that we are always fetching the model (model_requested must be true)
-        if (!request->model_requested)
-          {
-             CRI("Someone forgot to set model_requested for %" PRIu64 " to %" PRIu64 ".",
-                 request->offset, request->offset + request->length);
-             request->model_requested = EINA_TRUE;
-          }
         request->f = efl_model_children_slice_get(model, request->offset, request->length);
         request->f = efl_future_then(obj, request->f,
                                      .success = _model_fetched_cb,
@@ -917,11 +1017,11 @@ _batch_size_cb(void *data, Efl_Ui_Position_Manager_Size_Call_Config conf, Eina_R
 
    // Look in the temporary cache now for the beginning of the buffer
 #ifdef VIEWPORT_ENABLE
-   if (pd->viewport[0] && ((uint64_t)(conf.range.start_id + idx) < pd->viewport[0]->offset))
+   if (pd->viewport[0] && ((unsigned int)(conf.range.start_id + idx) < pd->viewport[0]->offset))
      {
-        while ((uint64_t)(conf.range.start_id + idx) < pd->viewport[0]->offset && idx < limit)
+        while ((unsigned int)(conf.range.start_id + idx) < pd->viewport[0]->offset && idx < limit)
           {
-             uint64_t search_index = conf.range.start_id + idx;
+             unsigned int search_index = conf.range.start_id + idx;
              requests = _cache_size_fetch(requests, &request, pd,
                                           search_index, &sizes[idx], item_base);
              idx++;
@@ -937,7 +1037,7 @@ _batch_size_cb(void *data, Efl_Ui_Position_Manager_Size_Call_Config conf, Eina_R
                (pd->viewport[i]->offset <= conf.range.start_id + idx) &&
                (conf.range.start_id + idx < (pd->viewport[i]->offset + pd->viewport[i]->count)))
           {
-             uint64_t offset = conf.range.start_id + idx - pd->viewport[i]->offset;
+             unsigned int offset = conf.range.start_id + idx - pd->viewport[i]->offset;
              Efl_Model *model = pd->viewport[i]->items[offset].model;
              Efl_Gfx_Entity *entity = pd->viewport[i]->items[offset].entity;
              Eina_Bool entity_request = EINA_FALSE;
@@ -993,7 +1093,7 @@ _batch_size_cb(void *data, Efl_Ui_Position_Manager_Size_Call_Config conf, Eina_R
    // Look in the temporary cache now for the end of the buffer
    while (idx < limit)
      {
-        uint64_t search_index = conf.range.start_id + idx;
+        unsigned int search_index = conf.range.start_id + idx;
         requests = _cache_size_fetch(requests, &request, pd,
                                      search_index, &sizes[idx], item_base);
         idx++;
@@ -1016,7 +1116,7 @@ _batch_size_cb(void *data, Efl_Ui_Position_Manager_Size_Call_Config conf, Eina_R
      {
         while (idx < limit)
           {
-             uint64_t search_index = conf.range.start_id + idx;
+             unsigned int search_index = conf.range.start_id + idx;
              requests = _cache_size_fetch(requests, &request, pd,
                                           search_index, &sizes[idx], item_base);
              idx++;
@@ -1057,11 +1157,11 @@ _batch_entity_cb(void *data, Efl_Ui_Position_Manager_Request_Range range, Eina_R
 
    // Look in the temporary cache now for the beginning of the buffer
 #ifdef VIEWPORT_ENABLE
-   if (pd->viewport[0] && ((uint64_t)(range.start_id + idx) < pd->viewport[0]->offset))
+   if (pd->viewport[0] && ((unsigned int)(range.start_id + idx) < pd->viewport[0]->offset))
      {
-        while (idx < limit && (uint64_t)(range.start_id + idx) < pd->viewport[0]->offset)
+        while (idx < limit && (unsigned int)(range.start_id + idx) < pd->viewport[0]->offset)
           {
-             uint64_t search_index = range.start_id + idx;
+             unsigned int search_index = range.start_id + idx;
 
              requests = _cache_entity_fetch(requests, &request, pd,
                                             search_index, &entities[idx]);
@@ -1079,7 +1179,7 @@ _batch_entity_cb(void *data, Efl_Ui_Position_Manager_Request_Range range, Eina_R
                (pd->viewport[i]->offset <= range.start_id + idx) &&
                (range.start_id + idx < (pd->viewport[i]->offset + pd->viewport[i]->count)))
           {
-             uint64_t offset = range.start_id + idx - pd->viewport[i]->offset;
+             unsigned int offset = range.start_id + idx - pd->viewport[i]->offset;
              Efl_Gfx_Entity *entity = pd->viewport[i]->items[offset].entity;
 
              if (!entity)
@@ -1106,7 +1206,7 @@ _batch_entity_cb(void *data, Efl_Ui_Position_Manager_Request_Range range, Eina_R
    // Look in the temporary cache now for the end of the buffer
    while (idx < limit)
      {
-        uint64_t search_index = range.start_id + idx;
+        unsigned int search_index = range.start_id + idx;
 
         requests = _cache_entity_fetch(requests, &request, pd,
                                        search_index, &entities[idx]);
@@ -1182,7 +1282,7 @@ _viewport_walk_fill(Eina_List *requests,
    for (j = 0; j < viewport->count; j++)
      {
         Efl_Ui_Collection_Item_Lookup *lookup;
-        uint64_t index = viewport->offset + j;
+        unsigned int index = viewport->offset + j;
 
         if (viewport->items[j].model) goto check_entity;
 
@@ -1314,7 +1414,7 @@ _manager_content_visible_range_changed_cb(void *data, const Efl_Event *ev)
    idx = lower_end;
    while (idx < upper_end)
      {
-        uint64_t search_index = idx;
+        unsigned int search_index = idx;
 
         requests = _cache_entity_fetch(requests, &request, pd,
                                        search_index, NULL);
@@ -1337,7 +1437,7 @@ _manager_content_visible_range_changed_cb(void *data, const Efl_Event *ev)
    Eina_List *requests = NULL;
    long baseid;
    unsigned int delta, marginup, margindown;
-   uint64_t upperlimit_offset, lowerlimit_offset;
+   unsigned int upperlimit_offset, lowerlimit_offset;
    unsigned int i;
 
    pd->start_id = event->start_id;
@@ -1443,7 +1543,7 @@ _manager_content_visible_range_changed_cb(void *data, const Efl_Event *ev)
         // We could optimize this to actually just move viewport around in most cases
         Efl_Ui_Collection_Item *items[3];
         unsigned int j = 0, t = 0;
-        uint64_t target, current;
+        unsigned int target, current;
 
         // Case where are at the top
         if (pd->start_id < delta && pd->viewport[0]->offset == 0) goto build_request;
@@ -1532,7 +1632,7 @@ _manager_content_visible_range_changed_cb(void *data, const Efl_Event *ev)
         request->offset = lowerlimit_offset;
         // This length work over multiple viewport as they are contiguous
         request->length = lowerlimit_offset - pd->viewport[0]->offset;
-        request->model_requested = EINA_TRUE;
+        request->need_size = EINA_TRUE;
         request->need_entity = EINA_TRUE;
 
         requests = eina_list_append(requests, request);
@@ -1549,7 +1649,7 @@ _manager_content_visible_range_changed_cb(void *data, const Efl_Event *ev)
         request->offset = upperlimit_offset;
         // This length work over multiple viewport as they are contiguous
         request->length = pd->viewport[2]->offset + pd->viewport[2]->count - upperlimit_offset;
-        request->model_requested = EINA_TRUE;
+        request->need_size = EINA_TRUE;
         request->need_entity = EINA_TRUE;
 
         requests = eina_list_append(requests, request);
@@ -1569,7 +1669,7 @@ EFL_CALLBACKS_ARRAY_DEFINE(manager_cbs,
 static void
 _item_scroll_internal(Eo *obj EINA_UNUSED,
                       Efl_Ui_Collection_View_Data *pd,
-                      uint64_t index,
+                      unsigned int index,
                       double align EINA_UNUSED,
                       Eina_Bool anim)
 {
@@ -1642,6 +1742,7 @@ _efl_ui_collection_view_position_manager_set(Eo *obj, Efl_Ui_Collection_View_Dat
           {
             case 1:
               efl_ui_position_manager_data_access_v1_data_access_set(pd->manager,
+                efl_provider_find(obj, EFL_UI_WIN_CLASS),
                 efl_ref(obj), _batch_entity_cb, _unref_cb,
                 efl_ref(obj), _batch_size_cb, _unref_cb,
                 count);
@@ -1661,8 +1762,27 @@ _efl_ui_collection_view_position_manager_get(const Eo *obj EINA_UNUSED,
 }
 
 static void
-_efl_model_count_changed(void *data EINA_UNUSED, const Efl_Event *event EINA_UNUSED)
+_efl_model_count_changed(void *data, const Efl_Event *event EINA_UNUSED)
 {
+   Efl_Ui_Collection_Request *request = NULL;
+   Eina_List *requests = NULL;
+   MY_DATA_GET(data, pd);
+   unsigned int index;
+   unsigned int count = 0;
+
+   count = efl_model_children_count_get(pd->model);
+   if (pd->focus.last)
+     {
+        index = _lookup_entity_index(pd->focus.last, NULL);
+
+        if (index + 1 == count)
+          return ;
+     }
+
+   // The last item is not the last item anymore
+   requests = _request_add(requests, &request, count, EINA_TRUE);
+   requests = _batch_request_flush(requests, data, pd);
+
    // We are not triggering efl_ui_position_manager_entity_data_access_set as it is can
    // only be slow, we rely on child added/removed instead (If we were to not rely on
    // child added/removed we could maybe use count changed)
@@ -1759,7 +1879,7 @@ _efl_model_child_added(void *data, const Efl_Event *event)
         if (!request) break;
         request->offset = ev->index;
         request->length = 1;
-        request->model_requested = EINA_TRUE;
+        request->need_size = EINA_TRUE;
         request->need_entity = EINA_TRUE;
 
         requests = eina_list_append(requests, request);
@@ -1786,7 +1906,7 @@ _efl_model_child_removed(void *data, const Efl_Event *event)
    unsigned int upper_end;
    long length;
    unsigned int count;
-   uint64_t request_length;
+   unsigned int request_length;
 
    // FIXME: later optimization, instead of reloading everyone, we could actually track index and self
    // update would be more efficient, but it is also more tricky
@@ -1839,7 +1959,7 @@ _efl_model_child_removed(void *data, const Efl_Event *event)
         if (!request) break;
         request->offset = pd->viewport[2]->offset + pd->viewport[i]->count - 1;
         request->length = 1;
-        request->model_requested = EINA_TRUE;
+        request->need_size = EINA_TRUE;
         request->need_entity = EINA_TRUE;
 
         requests = eina_list_append(requests, request);
@@ -1856,6 +1976,7 @@ _efl_model_child_removed(void *data, const Efl_Event *event)
    if (request_length > 0)
      {
         Efl_Ui_Collection_Request *request = NULL;
+
         requests = _request_add(requests, &request, ev->index, EINA_TRUE);
         request->length = request_length;
         requests = eina_list_append(requests, request);
@@ -1884,22 +2005,21 @@ _efl_ui_collection_view_model_changed(void *data, const Efl_Event *event)
    Efl_Model *mselect = NULL;
    Eina_Bool selection = EINA_FALSE, sizing = EINA_FALSE;
 
-   // Cleanup all object, pending request and refetch everything
+   // Cleanup all object, pending request to prepare refetching everything
    _all_cleanup(data, pd);
-
    if (pd->model) efl_event_callback_array_del(pd->model, model_cbs(), data);
-   efl_replace(&pd->model, NULL);
+   if (pd->multi_selectable_async_model)
+     {
+        efl_event_callback_forwarder_del(pd->multi_selectable_async_model,
+                                         EFL_UI_SELECTABLE_EVENT_SELECTION_CHANGED,
+                                         data);
+        efl_composite_detach(data, pd->multi_selectable_async_model);
+     }
 
    if (!ev->current)
      {
-        if (pd->multi_selectable_async_model)
-          {
-             efl_event_callback_forwarder_del(pd->multi_selectable_async_model,
-                                              EFL_UI_SELECTABLE_EVENT_SELECTION_CHANGED,
-                                              data);
-             efl_composite_detach(data, pd->multi_selectable_async_model);
-             efl_replace(&pd->multi_selectable_async_model, NULL);
-          }
+        efl_replace(&pd->model, NULL);
+        efl_replace(&pd->multi_selectable_async_model, NULL);
         return ;
      }
 
@@ -1921,11 +2041,11 @@ _efl_ui_collection_view_model_changed(void *data, const Efl_Event *event)
         // Search the composition of model for the one providing MULTI_SELECTABLE_ASYNC
         mselect = ev->current;
         while (mselect &&
-               !efl_isa(mselect, EFL_UI_MULTI_SELECTABLE_ASYNC_INTERFACE) &&
+               !efl_isa(mselect, EFL_UI_MULTI_SELECTABLE_INDEX_RANGE_INTERFACE) &&
                efl_isa(mselect, EFL_COMPOSITE_MODEL_CLASS))
           mselect = efl_ui_view_model_get(mselect);
 
-        if (!efl_isa(mselect, EFL_UI_MULTI_SELECTABLE_ASYNC_INTERFACE))
+        if (!efl_isa(mselect, EFL_UI_MULTI_SELECTABLE_INDEX_RANGE_INTERFACE))
           {
              mselect = NULL;
              selection = EINA_FALSE;
@@ -1938,15 +2058,9 @@ _efl_ui_collection_view_model_changed(void *data, const Efl_Event *event)
    // Build and connect the selection model properly
    if (!mselect)
      {
-        mselect = model = efl_add(EFL_UI_SELECT_MODEL_CLASS, data,
-                                  efl_ui_view_model_set(efl_added, model));
-     }
-   if (pd->multi_selectable_async_model)
-     {
-        efl_event_callback_forwarder_del(pd->multi_selectable_async_model,
-                                         EFL_UI_SELECTABLE_EVENT_SELECTION_CHANGED,
-                                         data);
-        efl_composite_detach(data, pd->multi_selectable_async_model);
+        mselect = model = efl_add_ref(EFL_UI_SELECT_MODEL_CLASS, data,
+                                      efl_ui_view_model_set(efl_added, model),
+                                      efl_loop_model_volatile_make(efl_added));
      }
    efl_replace(&pd->multi_selectable_async_model, mselect);
    efl_composite_attach(data, pd->multi_selectable_async_model);
@@ -1954,15 +2068,21 @@ _efl_ui_collection_view_model_changed(void *data, const Efl_Event *event)
                                     EFL_UI_SELECTABLE_EVENT_SELECTION_CHANGED,
                                     data);
 
-   if (!sizing) model = efl_add(EFL_UI_HOMOGENEOUS_MODEL_CLASS, data,
-                                efl_ui_view_model_set(efl_added, model));
+   if (!sizing) model = efl_add_ref(EFL_UI_HOMOGENEOUS_MODEL_CLASS, data,
+                                    efl_ui_view_model_set(efl_added, model),
+                                    efl_loop_model_volatile_make(efl_added));
+
+   efl_replace(&pd->model, model);
+   efl_event_callback_array_add(pd->model, model_cbs(), data);
+
+   if (mselect) efl_unref(mselect);
+   if (!sizing) efl_unref(model);
 
    count = efl_model_children_count_get(model);
 
 #ifdef VIEWPORT_ENABLE
    for (i = 0; i < 3; i++)
      {
-        Efl_Ui_Collection_Request *request;
 
         if (!pd->viewport[i]) continue ;
         if (pd->viewport[i]->count == 0) continue ;
@@ -1972,27 +2092,36 @@ _efl_ui_collection_view_model_changed(void *data, const Efl_Event *event)
 
         request->offset = pd->viewport[i]->offset;
         request->length = pd->viewport[i]->count;
-        request->model_requested = EINA_TRUE;
+        request->need_size = EINA_TRUE;
         request->need_entity = EINA_TRUE;
 
         requests = eina_list_append(requests, request);
      }
 #endif
+
+   // Fetch last item if necessary for later focus
+   if (efl_model_children_count_get(model))
+     {
+        Efl_Ui_Collection_Request *request = NULL;
+        uint64_t index = efl_model_children_count_get(model) - 1;
+
+        requests = _request_add(requests, &request, index, EINA_TRUE);
+     }
+
+   // Flush all pending request
    requests = _batch_request_flush(requests, data, pd);
 
-   pd->model = model;
-   efl_event_callback_array_add(pd->model, model_cbs(), data);
-
-   efl_ui_position_manager_entity_item_size_changed(pd->manager, 0, count - 1);
-   switch(efl_ui_position_manager_entity_version(pd->manager, 1))
+   switch (efl_ui_position_manager_entity_version(pd->manager, 1))
      {
        case 1:
          efl_ui_position_manager_data_access_v1_data_access_set(pd->manager,
+           efl_provider_find(data, EFL_UI_WIN_CLASS),
            efl_ref(data), _batch_entity_cb, _unref_cb,
            efl_ref(data), _batch_size_cb, _unref_cb,
            count);
        break;
      }
+   efl_ui_position_manager_entity_item_size_changed(pd->manager, 0, count - 1);
 }
 
 static void
@@ -2065,6 +2194,10 @@ _efl_ui_collection_view_efl_object_invalidate(Eo *obj,
 
    _all_cleanup(obj, pd);
 
+   //pd pan is given to edje, which reparents it, which forces us to manually deleting it
+   if (pd->pan)
+     efl_del(pd->pan);
+
    efl_invalidate(efl_super(obj, EFL_UI_COLLECTION_VIEW_CLASS));
 }
 
@@ -2186,11 +2319,11 @@ _efl_ui_collection_view_efl_ui_focus_manager_move(Eo *obj, Efl_Ui_Collection_Vie
      {
         Efl_Model *model;
         Eina_Value *vindex;
-        uint64_t index;
+        unsigned int index;
 
         model = efl_ui_view_model_get(new_obj);
         vindex = efl_model_property_get(model, "child.index");
-        if (eina_value_uint64_convert(vindex, &index))
+        if (eina_value_uint_convert(vindex, &index))
           _item_scroll_internal(obj, pd, index, .0, EINA_TRUE);
         eina_value_free(vindex);
      }
@@ -2233,25 +2366,22 @@ _efl_ui_collection_view_focus_manager_efl_ui_focus_manager_manager_focus_set(Eo
 {
    MY_DATA_GET(pd->collection, cpd);
    Efl_Ui_Item *item = NULL;
-   uint64_t item_id;
+   unsigned int item_id;
 
    if (focus == efl_ui_focus_manager_root_get(obj))
      {
         // Find last item
-        item_id = efl_model_children_count_get(cpd->model) - 1;
+        item = cpd->focus.previously;
+        if (!item) item = cpd->focus.last;
+        if (item) item_id = _lookup_entity_index(item, NULL);
+        else item_id = efl_model_children_count_get(cpd->model) - 1;
      }
    else
      {
-        Efl_Model *model;
-        Eina_Value *vindex;
-
         item = _find_item(obj, cpd, focus);
         if (!item) return ;
 
-        model = efl_ui_view_model_get(item);
-        vindex = efl_model_property_get(model, "child.index");
-        if (!eina_value_uint64_convert(vindex, &item_id)) return;
-        eina_value_free(vindex);
+        item_id = _lookup_entity_index(item, NULL);
      }
 
    // If this is NULL then we are before finalize, we cannot serve any sane value here
@@ -2265,7 +2395,7 @@ _efl_ui_collection_view_focus_manager_efl_ui_focus_manager_manager_focus_set(Eo
 }
 
 static int
-_id_from_item(Efl_Ui_Item *item, uint64_t *index)
+_id_from_item(Efl_Ui_Item *item, unsigned int *index)
 {
    Eina_Value *vindex;
    Efl_Model *model;
@@ -2273,7 +2403,7 @@ _id_from_item(Efl_Ui_Item *item, uint64_t *index)
    model = efl_ui_view_model_get(item);
 
    vindex = efl_model_property_get(model, "child.index");
-   EINA_SAFETY_ON_FALSE_RETURN_VAL(eina_value_uint64_convert(vindex, index), EINA_FALSE);
+   EINA_SAFETY_ON_FALSE_RETURN_VAL(eina_value_uint_convert(vindex, index), EINA_FALSE);
    eina_value_free(vindex);
    return EINA_TRUE;
 }
@@ -2282,8 +2412,9 @@ EOLIAN static Efl_Ui_Focus_Object *
 _efl_ui_collection_view_focus_manager_efl_ui_focus_manager_request_move(Eo *obj, Efl_Ui_Collection_View_Focus_Manager_Data *pd, Efl_Ui_Focus_Direction direction, Efl_Ui_Focus_Object *child, Eina_Bool logical)
 {
    MY_DATA_GET(pd->collection, cpd);
-   Efl_Ui_Item *new_item, *item;
-   uint64_t item_id;
+   Efl_Ui_Item *new_item = NULL;
+   Efl_Ui_Item *item;
+   unsigned int item_id;
 
    if (!child)
      child = efl_ui_focus_manager_focus_get(obj);
@@ -2291,20 +2422,20 @@ _efl_ui_collection_view_focus_manager_efl_ui_focus_manager_request_move(Eo *obj,
    item = _find_item(obj, cpd, child);
 
    //if this is NULL then we are before finalize, we cannot serve any sane value here
-   if (!cpd->manager) return NULL;
-   if (!item) return NULL;
+   if (!cpd->manager) goto end;
+   if (!item) goto end;
 
    if (!_id_from_item(item, &item_id))
-     return NULL;
+     goto end;
 
    if (ITEM_IS_OUTSIDE_VISIBLE(item_id))
      {
-        int new_id;
+        unsigned int new_id;
 
-        new_id = efl_ui_position_manager_entity_relative_item(cpd->manager,
-                                                              item_id,
-                                                              direction);
-        if (new_id == -1)
+        if (!efl_ui_position_manager_entity_relative_item(cpd->manager,
+                                                          item_id,
+                                                          direction,
+                                                          &new_id))
           {
              new_item = NULL;
           }
@@ -2328,9 +2459,9 @@ _efl_ui_collection_view_focus_manager_efl_ui_focus_manager_request_move(Eo *obj,
                   _assert_item_available(new_item, new_id, cpd);
                }
 #else
-               uint64_t search_index = new_id;
+               unsigned int search_index = new_id;
                lookup = (void*) eina_rbtree_inline_lookup(cpd->cache, &search_index,
-                                           sizeof (new_id), _cache_tree_lookup,
+                                           sizeof (search_index), _cache_tree_lookup,
                                            NULL);
                if (lookup)
                  {
@@ -2350,6 +2481,8 @@ _efl_ui_collection_view_focus_manager_efl_ui_focus_manager_request_move(Eo *obj,
         new_item = efl_ui_focus_manager_request_move(efl_super(obj, EFL_UI_COLLECTION_VIEW_FOCUS_MANAGER_CLASS), direction, child, logical);
      }
 
+ end:
+   efl_replace(&cpd->focus.previously, new_item);
    return new_item;
 }
 
