@@ -690,30 +690,10 @@ _elm_unneed_eldbus(void)
    eldbus_shutdown();
 }
 
-#ifdef ELM_ELOCATION
-static Eina_Bool _elm_need_elocation = EINA_FALSE;
-#endif
 EAPI Eina_Bool
 elm_need_elocation(void)
 {
-#ifdef ELM_ELOCATION
-   if (_elm_need_elocation) return EINA_TRUE;
-   _elm_need_elocation = EINA_TRUE;
-   elocation_init();
-   return EINA_TRUE;
-#else
    return EINA_FALSE;
-#endif
-}
-
-static void
-_elm_unneed_elocation(void)
-{
-#ifdef ELM_ELOCATION
-   if (!_elm_need_elocation) return;
-   _elm_need_elocation = EINA_FALSE;
-   elocation_shutdown();
-#endif
 }
 
 static Eina_Bool _elm_need_efreet = EINA_FALSE;
@@ -930,7 +910,6 @@ elm_quicklaunch_sub_shutdown(void)
         _elm_module_shutdown();
         if (_elm_prefs_initted)
           _elm_prefs_shutdown();
-        _efl_ui_dnd_shutdown();
         elm_color_class_shutdown();
      }
 
@@ -967,7 +946,6 @@ elm_quicklaunch_shutdown(void)
    _elm_unneed_efreet();
    _elm_unneed_e_dbus();
    _elm_unneed_eldbus();
-   _elm_unneed_elocation();
    _elm_unneed_ethumb();
    _elm_unneed_web();
 
@@ -1368,7 +1346,6 @@ elm_quicklaunch_exe_path_get(const char *exe, const char *cwd)
                }
              else
                {
-                  if (!*p) break;
                   p++;
                }
           }
