@@ -8,6 +8,7 @@
 # endif
 # ifdef HAVE_ELEMENTARY_WL2
 #  include <Ecore_Wl2.h>
+#  include "ecore_wl2_internal.h"
 # endif
 # ifdef HAVE_ELEMENTARY_DRM
 #  include <Ecore_Drm2.h>
@@ -23,6 +24,7 @@
 
 // Evas internal EO APIs
 # include "Evas.h"
+#define EFL_INTERNAL_UNSTABLE
 # include "Evas_Internal.h"
 # include "Emotion.h"
 
@@ -163,16 +165,8 @@
 
 # include "efl_ui_focus_parent_provider.eo.h"
 # include "efl_ui_focus_parent_provider_standard.eo.h"
-# include "efl_ui_selection_manager.eo.h"
 # include "efl_datetime_manager.eo.h"
 
-//FIXME this needs to not use elm_general.h and go to Efl_Ui.h
-# include <elm_general.h>
-# include <efl_text_interactive.eo.h>
-# include <efl_ui_text.eo.h>
-# include <efl_ui_text_editable.eo.h>
-# include <efl_ui_text_async.eo.h>
-
 extern const char *_efl_model_property_itemw;
 extern const char *_efl_model_property_itemh;
 extern const char *_efl_model_property_selfw;
@@ -276,7 +270,7 @@ struct _Efl_Ui_Theme_Data
  * the users config doesn't need to be wiped - simply new values need
  * to be put in
  */
-# define ELM_CONFIG_FILE_GENERATION 0x0016
+# define ELM_CONFIG_FILE_GENERATION 0x0018
 # define ELM_CONFIG_VERSION_EPOCH_OFFSET 16
 # define ELM_CONFIG_VERSION         ((ELM_CONFIG_EPOCH << ELM_CONFIG_VERSION_EPOCH_OFFSET) | \
                                      ELM_CONFIG_FILE_GENERATION)
@@ -328,6 +322,9 @@ extern const char *_elm_engines[];
 # define ELM_PRIV_SMART_CALLBACKS_DESC(name, signal, type) \
    {name, type},
 
+# define CLAMP(x, min, max) \
+   (((x) > (max)) ? (max) : (((x) < (min)) ? (min) : (x)))
+
 struct _Elm_Config_Flags
 {
    Eina_Bool engine : 1;
@@ -670,9 +667,6 @@ void                 _elm_prefs_data_init(void);
 void                 _elm_prefs_data_shutdown(void);
 
 /* init functions for dnd and cnp */
-Eo*                  _efl_ui_selection_manager_get(Eo *obj);
-void                 _efl_ui_dnd_shutdown(void);
-
 int                  _elm_ews_wm_init(void);
 void                 _elm_ews_wm_shutdown(void);
 void                 _elm_ews_wm_rescale(Elm_Theme *th,
@@ -854,6 +848,10 @@ void                 _elm_win_wl_cursor_set(Evas_Object *obj, const char *cursor
 void _efl_ui_focus_manager_redirect_events_del(Efl_Ui_Focus_Manager *manager, Eo *obj);
 void _efl_ui_focus_manager_redirect_events_add(Efl_Ui_Focus_Manager *manager, Eo *obj);
 
+EOAPI Eina_Bool efl_ui_focus_manager_calc_update_children(Eo *obj, Efl_Ui_Focus_Object *parent, Eina_List *children EFL_TRANSFER_OWNERSHIP);
+EOAPI void efl_ui_focus_manager_calc_update_order(Eo *obj, Efl_Ui_Focus_Object *parent, Eina_List *children EFL_TRANSFER_OWNERSHIP);
+
+
 void _efl_access_shutdown(void);
 
 /* Combobox: no proper support for Efl.Part API yet. */
@@ -1058,4 +1056,11 @@ typedef struct
 
 Efl_Ui_Shared_Win_Data* efl_ui_win_shared_data_get(Efl_Ui_Win *win);
 
+void _selection_changed_event_register(Eo *obj);
+void _selection_changed_event_unregister(Eo *obj);
+void _drop_event_register(Eo *obj);
+void _drop_event_unregister(Eo *obj);
+void _register_selection_changed(Efl_Ui_Selection *selection);
+Efl_Ui_Win* efl_ui_win_get(Evas_Object *obj);
+
 #endif
