@@ -6,7 +6,7 @@
 #include <Elementary.h>
 
 #include "elm_priv.h"
-#include "elm_genlist.eo.h"
+#include "elm_genlist_eo.h"
 
 #include <assert.h>
 
@@ -108,7 +108,8 @@ _item_content_get(void *data, Evas_Object *obj EINA_UNUSED, const char *part)
    if (!idata->item) return NULL;
 
    prop = eina_hash_find(idata->priv->connect.properties, part);
-   if (!prop) prop = part;
+   // If no property are connected, let's not try to guess randomly.
+   if (!prop) return NULL;
 
    value = efl_model_property_get(idata->model, prop);
    if (value == NULL) return NULL;
@@ -284,8 +285,8 @@ _efl_model_load_children(View_List_ItemData *pdata)
 
    f = efl_model_children_slice_get(pdata->priv->connect.model, 0,
                                     efl_model_children_count_get(pdata->priv->connect.model));
-   f = eina_future_then(f, _efl_model_load_children_then, pdata);
-   efl_future_Eina_FutureXXX_then(pdata->priv->genlist, f);
+   f = eina_future_then(f, _efl_model_load_children_then, pdata, NULL);
+   efl_future_then(pdata->priv->genlist, f);
 }
 
 static void
@@ -317,8 +318,8 @@ _efl_model_children_added_cb(void *data, const Efl_Event *event)
    Eina_Future *f;
 
    f = efl_model_children_slice_get(idata->priv->connect.model, evt->index, 1);
-   f = eina_future_then(f, _efl_model_load_children_then, idata);
-   efl_future_Eina_FutureXXX_then(idata->priv->genlist, f);
+   f = eina_future_then(f, _efl_model_load_children_then, idata, NULL);
+   efl_future_then(idata->priv->genlist, f);
 }
 
 static void
@@ -432,4 +433,4 @@ _elm_view_list_model_get(const Eo *obj EINA_UNUSED, Elm_View_List_Data *priv)
 {
    return priv->connect.model;
 }
-#include "elm_view_list.eo.c"
+#include "elm_view_list_eo.c"
