@@ -3,7 +3,7 @@
 
    @date 2000 (created)
 
-   @section toc Table of Contents
+   @section evas_toc Table of Contents
 
    @li @ref evas_main_intro
    @li @ref evas_main_work
@@ -33,7 +33,7 @@
    requirements.
 
    Evas can serve as a base for widget sets or toolkits
-   (e.g. Elementary, http://docs.enlightenment.org/auto/elementary/) by
+   (e.g. @ref elementary_main) by
    handling pixel drawing and regional change reporting, but does not
    manage windows itself, nor deal with input or window update event
    propagation.  In other words, it is intended for use in drawing
@@ -206,11 +206,10 @@ extern "C" {
 
 #include <interfaces/efl_gfx_types.eot.h>
 #include <interfaces/efl_gfx_path.eo.h>
-#include <interfaces/efl_input_types.eot.h>
 #include <interfaces/efl_text_types.eot.h>
+#include <canvas/efl_input_types.eot.h>
 #include <Evas_Common.h>
 
-
 #ifndef EFL_NOLEGACY_API_SUPPORT
 #include <Evas_Legacy.h>
 #endif
