@@ -30,7 +30,7 @@ _node_change(Efl_VG *obj, Efl_Canvas_Vg_Node_Data *nd)
         if (pnd->flags != EFL_GFX_CHANGE_FLAG_NONE) break;
         pnd->flags = EFL_GFX_CHANGE_FLAG_ALL;
      }
-   if (efl_invalidated_get(nd->vg_obj)) return;
+   if (!nd->vg_obj || efl_invalidated_get(nd->vg_obj)) return;
    efl_canvas_vg_object_change(nd->vd);
 }
 
@@ -391,29 +391,15 @@ _efl_canvas_vg_node_efl_object_parent_set(Eo *obj,
 static void
 _efl_canvas_vg_node_efl_gfx_stack_raise_to_top(Eo *obj, Efl_Canvas_Vg_Node_Data *pd EINA_UNUSED)
 {
-   Efl_Canvas_Vg_Container_Data *cd;
-   Eina_List *lookup, *next;
-   Eo *parent;
-
-   parent = efl_parent_get(obj);
-   if (!efl_isa(parent, EFL_CANVAS_VG_CONTAINER_CLASS)) goto on_error;
-   cd = efl_data_scope_get(parent, EFL_CANVAS_VG_CONTAINER_CLASS);
-
-   // FIXME: this could become slow with to much object
-   lookup = eina_list_data_find_list(cd->children, obj);
-   if (!lookup) goto on_error;
-
-   next = eina_list_next(lookup);
-   if (!next) return ;
-
-   cd->children = eina_list_remove_list(cd->children, lookup);
-   cd->children = eina_list_append_relative_list(cd->children, obj, next);
+   Efl_Canvas_Vg_Node *parent = efl_parent_get(obj);
+   if (!efl_isa(parent, EFL_CANVAS_VG_CONTAINER_CLASS)) return;
 
+   Efl_Canvas_Vg_Container_Data *cd = efl_data_scope_get(parent, EFL_CANVAS_VG_CONTAINER_CLASS);
+   if (!cd->children) return;
+   if (eina_list_data_get(eina_list_last(cd->children)) == obj) return;
+   cd->children = eina_list_remove(cd->children, obj);
+   cd->children = eina_list_append(cd->children, obj);
    _node_change(parent, efl_data_scope_get(parent, MY_CLASS));
-   return;
-
- on_error:
-   ERR("Err");
 }
 
 static void
@@ -479,29 +465,15 @@ _efl_canvas_vg_node_efl_gfx_stack_stack_below(Eo *obj,
 static void
 _efl_canvas_vg_node_efl_gfx_stack_lower_to_bottom(Eo *obj, Efl_Canvas_Vg_Node_Data *pd EINA_UNUSED)
 {
-   Efl_Canvas_Vg_Container_Data *cd;
-   Eina_List *lookup, *prev;
-   Eo *parent;
-
-   parent = efl_parent_get(obj);
-   if (!efl_isa(parent, EFL_CANVAS_VG_CONTAINER_CLASS)) goto on_error;
-   cd = efl_data_scope_get(parent, EFL_CANVAS_VG_CONTAINER_CLASS);
-
-   // FIXME: this could become slow with to much object
-   lookup = eina_list_data_find_list(cd->children, obj);
-   if (!lookup) goto on_error;
-
-   prev = eina_list_prev(lookup);
-   if (!prev) return;
-
-   cd->children = eina_list_remove_list(cd->children, lookup);
-   cd->children = eina_list_prepend_relative_list(cd->children, obj, prev);
+   Efl_Canvas_Vg_Node *parent = efl_parent_get(obj);
+   if (!efl_isa(parent, EFL_CANVAS_VG_CONTAINER_CLASS)) return;
 
+   Efl_Canvas_Vg_Container_Data *cd = efl_data_scope_get(parent, EFL_CANVAS_VG_CONTAINER_CLASS);
+   if (!cd->children) return;
+   if (eina_list_data_get(cd->children) == obj) return;
+   cd->children = eina_list_remove(cd->children, obj);
+   cd->children = eina_list_prepend(cd->children, obj);
    _node_change(parent, efl_data_scope_get(parent, MY_CLASS));
-   return;
-
- on_error:
-   ERR("Err");
 }
 
 static const Eo *
@@ -781,31 +753,31 @@ _efl_canvas_vg_node_efl_duplicate_duplicate(const Eo *obj, Efl_Canvas_Vg_Node_Da
 }
 
 EAPI Eina_Bool
-evas_vg_node_visible_get(Eo *obj)
+evas_vg_node_visible_get(Evas_Vg_Node *obj)
 {
    return efl_gfx_entity_visible_get(obj);
 }
 
 EAPI void
-evas_vg_node_visible_set(Eo *obj, Eina_Bool v)
+evas_vg_node_visible_set(Evas_Vg_Node *obj, Eina_Bool v)
 {
    efl_gfx_entity_visible_set(obj, v);
 }
 
 EAPI void
-evas_vg_node_color_get(Eo *obj, int *r, int *g, int *b, int *a)
+evas_vg_node_color_get(Evas_Vg_Node *obj, int *r, int *g, int *b, int *a)
 {
    efl_gfx_color_get(obj, r, g, b, a);
 }
 
 EAPI void
-evas_vg_node_color_set(Eo *obj, int r, int g, int b, int a)
+evas_vg_node_color_set(Evas_Vg_Node *obj, int r, int g, int b, int a)
 {
    efl_gfx_color_set(obj, r, g, b, a);
 }
 
 EAPI void
-evas_vg_node_geometry_get(Eo *obj, int *x, int *y, int *w, int *h)
+evas_vg_node_geometry_get(Evas_Vg_Node *obj, int *x, int *y, int *w, int *h)
 {
    Eina_Rect r;
    r.pos = efl_gfx_entity_position_get(obj);
@@ -817,32 +789,32 @@ evas_vg_node_geometry_get(Eo *obj, int *x, int *y, int *w, int *h)
 }
 
 EAPI void
-evas_vg_node_geometry_set(Eo *obj, int x, int y, int w, int h)
+evas_vg_node_geometry_set(Evas_Vg_Node *obj, int x, int y, int w, int h)
 {
    efl_gfx_entity_position_set(obj, EINA_POSITION2D(x, y));
    efl_gfx_entity_size_set(obj, EINA_SIZE2D(w,  h));
 }
 
 EAPI void
-evas_vg_node_stack_below(Eo *obj, Eo *below)
+evas_vg_node_stack_below(Evas_Vg_Node *obj, Eo *below)
 {
    efl_gfx_stack_below(obj, below);
 }
 
 EAPI void
-evas_vg_node_stack_above(Eo *obj, Eo *above)
+evas_vg_node_stack_above(Evas_Vg_Node *obj, Eo *above)
 {
    efl_gfx_stack_above(obj, above);
 }
 
 EAPI void
-evas_vg_node_raise(Eo *obj)
+evas_vg_node_raise(Evas_Vg_Node *obj)
 {
    efl_gfx_stack_raise_to_top(obj);
 }
 
 EAPI void
-evas_vg_node_lower(Eo *obj)
+evas_vg_node_lower(Evas_Vg_Node *obj)
 {
    efl_gfx_stack_lower_to_bottom(obj);
 }
