@@ -2,14 +2,12 @@
 #include "evas_common_private.h"
 #include "evas_private.h"
 
-#define EFL_INTERNAL_UNSTABLE
-#include "interfaces/efl_common_internal.h"
-
 EVAS_MEMPOOL(_mp_pc);
 
 extern Eina_Hash* signals_hash_table;
 
 /* Legacy events, do not use anywhere */
+void _evas_object_smart_callback_call_internal(Evas_Object *eo_obj, const Efl_Event_Description *efl_event_desc);
 static const Efl_Event_Description _EVAS_OBJECT_EVENT_FREE = EFL_EVENT_DESCRIPTION("free");
 static const Efl_Event_Description _EVAS_OBJECT_EVENT_DEL = EFL_EVENT_DESCRIPTION("del");
 #define EVAS_OBJECT_EVENT_FREE (&(_EVAS_OBJECT_EVENT_FREE))
@@ -347,17 +345,6 @@ evas_event_callback_call(Evas *eo_e, Evas_Callback_Type type, void *event_info)
    efl_event_callback_legacy_call(eo_e, _legacy_evas_callback_table(type), event_info);
 }
 
-static void
-_evas_smart_callback_legacy_git_er_done(Evas_Object *eo_obj, const Efl_Event_Description *efl_event_desc)
-{
-   if (efl_event_desc == EFL_GFX_ENTITY_EVENT_POSITION_CHANGED)
-     evas_object_smart_callback_call(eo_obj, "move", NULL);
-   else if (efl_event_desc == EFL_GFX_ENTITY_EVENT_SIZE_CHANGED)
-     evas_object_smart_callback_call(eo_obj, "resize", NULL);
-   else if (efl_event_desc == EFL_GFX_ENTITY_EVENT_STACKING_CHANGED)
-     evas_object_smart_callback_call(eo_obj, "restack", NULL);
-}
-
 static void
 _evas_callback_legacy_smart_compatibility_do_it(Evas_Object *eo_obj, const Efl_Event_Description *efl_event_desc, void *event_info)
 {
@@ -406,9 +393,10 @@ evas_object_event_callback_call(Evas_Object *eo_obj, Evas_Object_Protected_Data
         type == EVAS_CALLBACK_MULTI_DOWN ||
         type == EVAS_CALLBACK_MOUSE_UP ||
         type == EVAS_CALLBACK_MULTI_UP)
-     _efl_canvas_gesture_manager_filter_event(e->gesture_manager, eo_obj, event_info);
+     _efl_canvas_gesture_manager_filter_event(e->gmd, eo_obj, event_info);
 
-   _evas_smart_callback_legacy_git_er_done(eo_obj, efl_event_desc);
+   if (obj->is_smart)
+     _evas_object_smart_callback_call_internal(eo_obj, efl_event_desc);
 
    if (!_evas_object_callback_has_by_type(obj, type))
      goto nothing_here;
@@ -829,17 +817,12 @@ void
 evas_object_callbacks_event_catcher_add(Eo *eo_obj EINA_UNUSED, Evas_Object_Protected_Data *obj, const Efl_Callback_Array_Item *array)
 {
    Evas_Callback_Type type = EVAS_CALLBACK_LAST;
-   void *gd = NULL;
    int i;
 
    for (i = 0; array[i].desc != NULL; i++)
      {
         if (obj->layer && obj->layer->evas && obj->layer->evas->gesture_manager)
-          {
-             if (!gd) gd = _efl_canvas_gesture_manager_private_data_get(obj->layer->evas->gesture_manager);
-
-             _efl_canvas_gesture_manager_callback_add_hook(gd, obj->object, array[i].desc);
-          }
+          _efl_canvas_gesture_manager_callback_add_hook(obj->layer->evas->gmd, obj->object, array[i].desc);
 
         if (array[i].desc == EFL_CANVAS_OBJECT_EVENT_ANIMATOR_TICK)
           {
@@ -867,7 +850,6 @@ evas_object_callbacks_event_catcher_add(Eo *eo_obj EINA_UNUSED, Evas_Object_Prot
 void
 evas_object_callbacks_event_catcher_del(Eo *eo_obj EINA_UNUSED, Evas_Object_Protected_Data *obj, const Efl_Callback_Array_Item *array)
 {
-   void *gd = NULL;
    int i;
 
    if (!obj->layer ||
@@ -877,11 +859,7 @@ evas_object_callbacks_event_catcher_del(Eo *eo_obj EINA_UNUSED, Evas_Object_Prot
    for (i = 0; array[i].desc != NULL; i++)
      {
         if (obj->layer->evas->gesture_manager)
-          {
-             if (!gd) gd = _efl_canvas_gesture_manager_private_data_get(obj->layer->evas->gesture_manager);
-
-             _efl_canvas_gesture_manager_callback_del_hook(gd, obj->object, array[i].desc);
-          }
+          _efl_canvas_gesture_manager_callback_del_hook(obj->layer->evas->gmd, obj->object, array[i].desc);
 
         if (array[i].desc == EFL_CANVAS_OBJECT_EVENT_ANIMATOR_TICK)
           {
@@ -894,3 +872,10 @@ evas_object_callbacks_event_catcher_del(Eo *eo_obj EINA_UNUSED, Evas_Object_Prot
      }
 }
 
+void
+evas_object_callbacks_shutdown(Eo *eo_obj EINA_UNUSED, Evas_Object_Protected_Data *obj)
+{
+   if (obj->animator_ref > 0)
+     efl_event_callback_del(obj->layer->evas->evas, EFL_CANVAS_OBJECT_EVENT_ANIMATOR_TICK, _animator_repeater, obj);
+   obj->animator_ref = 0;
+}
