@@ -412,18 +412,16 @@ _evas_box_efl_canvas_group_group_add(Eo *eo_obj, Evas_Object_Box_Data *priv)
 EOLIAN static void
 _evas_box_efl_canvas_group_group_del(Eo *o, Evas_Object_Box_Data *priv)
 {
-   Eina_List *l;
-
-   l = priv->children;
-   while (l)
+   while (priv->children)
      {
-        Evas_Object_Box_Option *opt = l->data;
+        Evas_Object_Box_Option *opt = eina_list_data_get(priv->children);
 
         _evas_object_box_child_callbacks_unregister(opt->obj, o);
         evas_obj_box_internal_option_free(o, opt);
-        l = eina_list_remove_list(l, l);
+        priv->children = eina_list_remove_list(priv->children, priv->children);
      }
 
+
    if (priv->layout.data && priv->layout.free_data)
      priv->layout.free_data(priv->layout.data);
 
@@ -447,7 +445,9 @@ _evas_box_efl_gfx_entity_position_set(Eo *o, Evas_Object_Box_Data *_pd EINA_UNUS
    if (_evas_object_intercept_call(o, EVAS_OBJECT_INTERCEPT_CB_MOVE , 0, pos.x, pos.y))
      return;
 
-   efl_gfx_entity_position_set(cso->clipper, pos);
+   if (!evas_object_static_clip_get(cso->clipper))
+     efl_gfx_entity_position_set(cso->clipper, pos);
+
    /* this skips the call to _evas_object_smart_clipped_smart_move_internal
     * since box internals will automatically recalc all the child positions
     * at a later point
