@@ -13,7 +13,7 @@ EVAS_MEMPOOL(_mp_sh);
 
 /* evas internal stuff */
 static const Evas_Object_Proxy_Data default_proxy = {
-  NULL, NULL, NULL, 0, 0, NULL, 0, 0, 0, 0
+  NULL, NULL, 0, 0, NULL, 0, 0, 0, 0
 };
 static const Evas_Object_Map_Data default_map = {
   { NULL, NULL, 0, 0 }, { NULL, NULL, 0, 0 }, NULL, 0, 0, NULL, NULL
@@ -34,7 +34,6 @@ static const Evas_Object_Events_Data default_events = {
 Eina_Cow *evas_object_proxy_cow = NULL;
 Eina_Cow *evas_object_map_cow = NULL;
 Eina_Cow *evas_object_state_cow = NULL;
-Eina_Cow *evas_object_3d_cow = NULL;
 Eina_Cow *evas_object_mask_cow = NULL;
 Eina_Cow *evas_object_events_cow = NULL;
 
@@ -49,29 +48,26 @@ typedef struct _Event_Animation
 static Eina_Bool
 _init_cow(void)
 {
-   if (evas_object_map_cow && evas_object_proxy_cow && evas_object_state_cow && evas_object_3d_cow) return EINA_TRUE;
+   if (evas_object_map_cow && evas_object_proxy_cow && evas_object_state_cow) return EINA_TRUE;
 
    evas_object_proxy_cow = eina_cow_add("Evas Object Proxy", sizeof (Evas_Object_Proxy_Data), 8, &default_proxy, EINA_TRUE);
    evas_object_map_cow = eina_cow_add("Evas Object Map", sizeof (Evas_Object_Map_Data), 8, &default_map, EINA_TRUE);
    evas_object_state_cow = eina_cow_add("Evas Object State", sizeof (Evas_Object_Protected_State), 64, &default_state, EINA_FALSE);
-   evas_object_3d_cow = eina_cow_add("Evas Object 3D", sizeof (Evas_Object_3D_Data), 8, &default_proxy, EINA_TRUE);
    evas_object_mask_cow = eina_cow_add("Evas Mask Data", sizeof (Evas_Object_Mask_Data), 8, &default_mask, EINA_TRUE);
    evas_object_events_cow = eina_cow_add("Evas Events Data", sizeof (Evas_Object_Events_Data), 8, &default_events, EINA_TRUE);
 
    if (!(evas_object_map_cow && evas_object_proxy_cow && evas_object_state_cow &&
-         evas_object_3d_cow && evas_object_mask_cow && evas_object_events_cow))
+         evas_object_mask_cow && evas_object_events_cow))
      {
         eina_cow_del(evas_object_proxy_cow);
         eina_cow_del(evas_object_map_cow);
         eina_cow_del(evas_object_state_cow);
-        eina_cow_del(evas_object_3d_cow);
         eina_cow_del(evas_object_mask_cow);
         eina_cow_del(evas_object_events_cow);
 
         evas_object_proxy_cow = NULL;
         evas_object_map_cow = NULL;
         evas_object_state_cow = NULL;
-        evas_object_3d_cow = NULL;
         evas_object_mask_cow = NULL;
         evas_object_events_cow = NULL;
 
@@ -81,9 +77,9 @@ _init_cow(void)
    return EINA_TRUE;
 }
 
-static Evas_Object_Pointer_Data *
-_evas_object_pointer_data_find(Evas_Object_Protected_Data *obj,
-                               Efl_Input_Device *pointer)
+Evas_Object_Pointer_Data *
+evas_object_pointer_data_find(Evas_Object_Protected_Data *obj,
+                              Efl_Input_Device *pointer)
 {
    Evas_Object_Pointer_Data *pdata;
 
@@ -95,22 +91,6 @@ _evas_object_pointer_data_find(Evas_Object_Protected_Data *obj,
    return NULL;
 }
 
-static void
-_evas_object_pointer_grab_del(Evas_Object_Protected_Data *obj, Evas_Object_Pointer_Data *pdata);
-
-static void
-_evas_device_del_cb(void *data, const Efl_Event *ev)
-{
-   Evas_Object_Protected_Data *obj;
-   Evas_Object_Pointer_Data *pdata;
-
-   obj = efl_data_scope_safe_get(data, MY_CLASS);
-   EINA_SAFETY_ON_NULL_RETURN(obj);
-   pdata = _evas_object_pointer_data_find(obj, ev->object);
-   if (!pdata) return;
-   _evas_object_pointer_grab_del(obj, pdata);
-}
-
 static void
 _evas_object_proxy_grab_del(Evas_Object_Protected_Data *obj,
                             Evas_Object_Pointer_Data *pdata)
@@ -138,9 +118,9 @@ _evas_object_proxy_grab_del(Evas_Object_Protected_Data *obj,
      }
 }
 
-static void
-_evas_object_pointer_grab_del(Evas_Object_Protected_Data *obj,
-                              Evas_Object_Pointer_Data *pdata)
+void
+evas_object_pointer_grab_del(Evas_Object_Protected_Data *obj,
+                             Evas_Object_Pointer_Data *pdata)
 {
    if ((pdata->mouse_grabbed > 0) && (obj->layer) && (obj->layer->evas))
      pdata->evas_pdata->seat->mouse_grabbed -= pdata->mouse_grabbed;
@@ -151,11 +131,12 @@ _evas_object_pointer_grab_del(Evas_Object_Protected_Data *obj,
         if (obj->proxy->is_proxy && obj->proxy->src_events)
           _evas_object_proxy_grab_del(obj, pdata);
      }
-   efl_event_callback_del(pdata->evas_pdata->pointer, EFL_EVENT_DEL,
-                          _evas_device_del_cb, obj->object);
-   EINA_COW_WRITE_BEGIN(evas_object_events_cow, obj->events, Evas_Object_Events_Data, events)
-     events->pointer_grabs = eina_inlist_remove(events->pointer_grabs, EINA_INLIST_GET(pdata));
-   EINA_COW_WRITE_END(evas_object_events_cow, obj->events, events);
+   if (obj->events->pointer_grabs)
+     {
+        EINA_COW_WRITE_BEGIN(evas_object_events_cow, obj->events, Evas_Object_Events_Data, events)
+          events->pointer_grabs = eina_inlist_remove(events->pointer_grabs, EINA_INLIST_GET(pdata));
+        EINA_COW_WRITE_END(evas_object_events_cow, obj->events, events);
+     }
 
    free(pdata);
 }
@@ -170,14 +151,13 @@ _evas_object_pointer_data_add(Evas_Pointer_Data *evas_pdata,
    EINA_SAFETY_ON_NULL_RETURN_VAL(pdata, NULL);
    pdata->pointer_mode = EVAS_OBJECT_POINTER_MODE_AUTOGRAB;
    pdata->evas_pdata = evas_pdata;
+   pdata->obj = obj;
    EINA_COW_WRITE_BEGIN(evas_object_events_cow, obj->events, Evas_Object_Events_Data, events)
      events->pointer_grabs = eina_inlist_append(events->pointer_grabs,
                                                 EINA_INLIST_GET(pdata));
    EINA_COW_WRITE_END(evas_object_events_cow, obj->events, events);
 
-   efl_event_callback_priority_add(evas_pdata->pointer, EFL_EVENT_DEL,
-                                   EFL_CALLBACK_PRIORITY_BEFORE,
-                                   _evas_device_del_cb, obj->object);
+   efl_input_device_grab_register(evas_pdata->pointer, obj->object, pdata);
    return pdata;
 }
 
@@ -187,7 +167,9 @@ _evas_object_pointer_data_get(Evas_Pointer_Data *evas_pdata,
 {
    Evas_Object_Pointer_Data *pdata;
 
-   pdata = _evas_object_pointer_data_find(obj, evas_pdata->pointer);
+   if (!obj) return NULL;
+
+   pdata = evas_object_pointer_data_find(obj, evas_pdata->pointer);
 
    //The pointer does not exist yet - create one.
    if (!pdata)
@@ -220,7 +202,6 @@ _efl_canvas_object_efl_object_constructor(Eo *eo_obj, Evas_Object_Protected_Data
    obj->map = eina_cow_alloc(evas_object_map_cow);
    obj->cur = eina_cow_alloc(evas_object_state_cow);
    obj->prev = eina_cow_alloc(evas_object_state_cow);
-   obj->data_3d = eina_cow_alloc(evas_object_3d_cow);
    obj->mask = eina_cow_alloc(evas_object_mask_cow);
    obj->events = eina_cow_alloc(evas_object_events_cow);
 
@@ -258,15 +239,6 @@ end:
    return efl_finalize(efl_super(eo_obj, MY_CLASS));
 }
 
-EOLIAN const Efl_Canvas_Gesture_Manager *
-_efl_canvas_object_gesture_manager_get(Eo *eo_obj EINA_UNUSED, Evas_Object_Protected_Data *pd)
-{
-   if (!pd->layer || !pd->layer->evas)
-     return NULL;
-
-   return (pd->layer->evas)->gesture_manager;
-}
-
 void
 evas_object_change_reset(Evas_Object_Protected_Data *obj)
 {
@@ -542,7 +514,6 @@ evas_object_free(Evas_Object_Protected_Data *obj, Eina_Bool clean_layer)
    eina_cow_free(evas_object_map_cow, (const Eina_Cow_Data**) &obj->map);
    eina_cow_free(evas_object_state_cow, (const Eina_Cow_Data**) &obj->cur);
    eina_cow_free(evas_object_state_cow, (const Eina_Cow_Data**) &obj->prev);
-   eina_cow_free(evas_object_3d_cow, (const Eina_Cow_Data**) &obj->data_3d);
    eina_cow_free(evas_object_mask_cow, (const Eina_Cow_Data**) &obj->mask);
    eina_cow_free(evas_object_events_cow, (const Eina_Cow_Data**) &obj->events);
 
@@ -562,7 +533,6 @@ evas_object_change(Evas_Object *eo_obj EINA_UNUSED, Evas_Object_Protected_Data *
    Evas_Object_Protected_Data *obj2;
    Evas_Object *eo_obj2;
    Eina_Bool movch = EINA_FALSE;
-   Evas_Canvas3D_Texture *texture;
 
    if ((!obj->layer) || (!obj->layer->evas)) return;
    if (obj->layer->evas->nochange) return;
@@ -593,10 +563,6 @@ evas_object_change(Evas_Object *eo_obj EINA_UNUSED, Evas_Object_Protected_Data *
         if (!obj2) continue;
         evas_object_change(eo_obj2, obj2);
      }
-   EINA_LIST_FOREACH(obj->proxy->proxy_textures, l, texture)
-     {
-        evas_canvas3d_object_change(texture, EVAS_CANVAS3D_STATE_TEXTURE_DATA, NULL);
-     }
    if (obj->smart.parent)
      {
         Evas_Object_Protected_Data *smart_parent = efl_data_scope_get(obj->smart.parent, MY_CLASS);
@@ -1068,29 +1034,39 @@ _efl_canvas_object_efl_object_invalidate(Eo *eo_obj, Evas_Object_Protected_Data
    evas_object_hide(eo_obj);
 
    if (obj->events)
-     EINA_COW_WRITE_BEGIN(evas_object_events_cow, obj->events, Evas_Object_Events_Data, events)
-       {
-          Evas_Public_Data *edata = NULL;
+     {
+        Eina_Inlist *pointer_grabs;
 
-          if (!efl_invalidated_get(evas_object_evas_get(eo_obj)))
-            edata = efl_data_scope_get(evas_object_evas_get(eo_obj), EVAS_CANVAS_CLASS);
+        EINA_COW_WRITE_BEGIN(evas_object_events_cow, obj->events, Evas_Object_Events_Data, events)
+          {
+             Evas_Public_Data *edata = NULL;
 
-          EINA_LIST_FREE (events->focused_by_seats, dev)
-            {
-               event_id = _evas_event_counter;
-               efl_event_callback_del(dev, EFL_EVENT_INVALIDATE,
-                                      _evas_focus_device_invalidate_cb, obj);
-               if (edata) eina_hash_del_by_key(edata->focused_objects, &dev);
-               _evas_focus_dispatch_event(obj, dev, EINA_FALSE);
-               if ((obj->layer) && (obj->layer->evas))
-                 _evas_post_event_callback_call(obj->layer->evas->evas, obj->layer->evas, event_id);
-            }
-          EINA_INLIST_FREE(events->pointer_grabs, pdata)
-            _evas_object_pointer_grab_del(obj, pdata);
-          EINA_LIST_FREE(events->events_whitelist, dev)
-            efl_event_callback_del(dev, EFL_EVENT_DEL, _whitelist_events_device_remove_cb, obj);
-       }
-     EINA_COW_WRITE_END(evas_object_events_cow, obj->events, events);
+             if (!efl_invalidated_get(evas_object_evas_get(eo_obj)))
+               edata = efl_data_scope_get(evas_object_evas_get(eo_obj), EVAS_CANVAS_CLASS);
+
+             EINA_LIST_FREE (events->focused_by_seats, dev)
+               {
+                  event_id = _evas_event_counter;
+                  efl_event_callback_del(dev, EFL_EVENT_INVALIDATE,
+                                         _evas_focus_device_invalidate_cb, obj);
+                  if (edata) eina_hash_del_by_key(edata->focused_objects, &dev);
+                  _evas_focus_dispatch_event(obj, dev, EINA_FALSE);
+                  if ((obj->layer) && (obj->layer->evas))
+                    _evas_post_event_callback_call(obj->layer->evas->evas, obj->layer->evas, event_id);
+               }
+             pointer_grabs = events->pointer_grabs;
+             events->pointer_grabs = NULL;
+             EINA_LIST_FREE(events->events_whitelist, dev)
+               efl_event_callback_del(dev, EFL_EVENT_DEL, _whitelist_events_device_remove_cb, obj);
+          }
+        EINA_COW_WRITE_END_NOGC(evas_object_events_cow, obj->events, events);
+
+        EINA_INLIST_FREE(pointer_grabs, pdata)
+          {
+             pointer_grabs = eina_inlist_remove(pointer_grabs, EINA_INLIST_GET(pdata));
+             efl_input_device_grab_unregister(pdata->evas_pdata->pointer, eo_obj, pdata);
+          }
+     }
 
    event_id = _evas_object_event_new();
    evas_object_event_callback_call(eo_obj, obj, EVAS_CALLBACK_DEL, NULL, event_id, NULL);
@@ -1116,21 +1092,6 @@ _efl_canvas_object_efl_object_invalidate(Eo *eo_obj, Evas_Object_Protected_Data
              if (efl_isa(proxy, EFL_GFX_FILTER_INTERFACE))
                efl_gfx_filter_source_set(proxy, NULL, eo_obj);
           }
-
-        /* Eina_Cow has no way to know if we are going to really change something
-           or not. So before calling the cow, let's check if we want to do something */
-        if (obj->proxy->proxy_textures)
-          {
-             EINA_COW_WRITE_BEGIN(evas_object_proxy_cow, obj->proxy,
-                                  Evas_Object_Proxy_Data, proxy_src)
-               {
-                  Evas_Canvas3D_Texture *texture;
-
-                  EINA_LIST_FREE(proxy_src->proxy_textures, texture)
-                    evas_canvas3d_texture_source_set(texture, NULL);
-               }
-             EINA_COW_WRITE_END(evas_object_proxy_cow, obj->proxy, proxy_src);
-          }
      }
 
    if (obj->cur)
@@ -1185,7 +1146,7 @@ _efl_canvas_object_efl_object_destructor(Eo *eo_obj, Evas_Object_Protected_Data
 end:
    evas_object_event_callback_all_del(eo_obj);
    evas_object_event_callback_cleanup(eo_obj);
-
+   evas_object_callbacks_shutdown(eo_obj, obj);
    efl_destructor(efl_super(eo_obj, MY_CLASS));
 }
 
@@ -1471,7 +1432,7 @@ _efl_canvas_object_efl_gfx_hint_hint_size_restricted_max_set(Eo *eo_obj, Evas_Ob
         if (!sz.w && !sz.h) return;
         _evas_object_size_hint_alloc(eo_obj, obj);
      }
-   if ((obj->size_hints->max.w == sz.w) && (obj->size_hints->max.h == sz.h)) return;
+   if (EINA_SIZE2D_EQ(obj->size_hints->max, sz)) return;
    obj->size_hints->max = sz;
    if ((obj->size_hints->max.w != -1) && (obj->size_hints->max.w < obj->size_hints->min.w))
      ERR("restricted max width hint is now smaller than restricted min width hint! (%d < %d)", obj->size_hints->max.w, obj->size_hints->min.w);
@@ -1502,7 +1463,7 @@ _efl_canvas_object_efl_gfx_hint_hint_size_restricted_min_set(Eo *eo_obj, Evas_Ob
         if (!sz.w && !sz.h) return;
         _evas_object_size_hint_alloc(eo_obj, obj);
      }
-   if ((obj->size_hints->min.w == sz.w) && (obj->size_hints->min.h == sz.h)) return;
+   if (EINA_SIZE2D_EQ(obj->size_hints->min, sz)) return;
    obj->size_hints->min = sz;
    if ((obj->size_hints->max.w != -1) && (obj->size_hints->max.w < obj->size_hints->min.w))
      ERR("restricted max width hint is now smaller than restricted min width hint! (%d < %d)", obj->size_hints->max.w, obj->size_hints->min.w);
@@ -1588,7 +1549,7 @@ _efl_canvas_object_efl_gfx_hint_hint_size_max_set(Eo *eo_obj, Evas_Object_Protec
         if ((sz.w == -1) && (sz.h == -1)) return;
         _evas_object_size_hint_alloc(eo_obj, obj);
      }
-   if ((obj->size_hints->user_max.w == sz.w) && (obj->size_hints->user_max.h == sz.h)) return;
+   if (EINA_SIZE2D_EQ(obj->size_hints->user_max, sz)) return;
    obj->size_hints->user_max.w = sz.w;
    obj->size_hints->user_max.h = sz.h;
 
@@ -1652,7 +1613,7 @@ _efl_canvas_object_efl_gfx_hint_hint_size_min_set(Eo *eo_obj, Evas_Object_Protec
         if (!sz.w && !sz.h) return;
         _evas_object_size_hint_alloc(eo_obj, obj);
      }
-   if ((obj->size_hints->user_min.w == sz.w) && (obj->size_hints->user_min.h == sz.h)) return;
+   if (EINA_SIZE2D_EQ(obj->size_hints->user_min, sz)) return;
    obj->size_hints->user_min = sz;
    evas_object_inform_call_changed_size_hints(eo_obj, obj);
 }
