@@ -266,7 +266,8 @@ enum _Shader_Type {
    SHD_FILTER_CURVE,
    SHD_FILTER_BLUR_X,
    SHD_FILTER_BLUR_Y,
-   SHD_FILTER_GRAYSCALE
+   SHD_FILTER_GRAYSCALE,
+   SHD_FILTER_INVERSE_COLOR
 };
 
 #define ARRAY_BUFFER_USE 500
@@ -682,6 +683,7 @@ void              evas_gl_common_filter_blur_push(Evas_Engine_GL_Context *gc, Ev
                                                   double dx, double dy, double dw, double dh, const double * const values, const double * const offsets, int count, double radius,
                                                   Eina_Bool horiz, Eina_Bool alphaonly);
 void              evas_gl_common_filter_grayscale_push(Evas_Engine_GL_Context *gc, Evas_GL_Texture *tex, int x, int y, int w, int h);
+void              evas_gl_common_filter_inverse_color_push(Evas_Engine_GL_Context *gc, Evas_GL_Texture *tex, int x, int y, int w, int h);
 
 int               evas_gl_common_shader_program_init(Evas_GL_Shared *shared);
 void              evas_gl_common_shader_program_shutdown(Evas_GL_Shared *shared);
@@ -734,6 +736,7 @@ void              evas_gl_common_texture_rgb_a_pair_update(Evas_GL_Texture *tex,
 Evas_Colorspace   evas_gl_common_gl_format_to_colorspace(GLuint f);
 
 void              evas_gl_common_image_alloc_ensure(Evas_GL_Image *im);
+Evas_GL_Image    *evas_gl_common_image_load(Evas_Engine_GL_Context *gc, const char *file, const char *key, Evas_Image_Load_Opts *lo, int *error);
 Evas_GL_Image    *evas_gl_common_image_mmap(Evas_Engine_GL_Context *gc, Eina_File *f, const char *key, Evas_Image_Load_Opts *lo, int *error);
 Evas_GL_Image    *evas_gl_common_image_new_from_copied_data(Evas_Engine_GL_Context *gc, unsigned int w, unsigned int h, DATA32 *data, int alpha, Evas_Colorspace cspace);
 Evas_GL_Image    *evas_gl_common_image_new(Evas_Engine_GL_Context *gc, unsigned int w, unsigned int h, int alpha, Evas_Colorspace cspace);
@@ -976,8 +979,6 @@ _comp_tex_sub_2d(Evas_Engine_GL_Context *gc, int x, int y, int w, int h, int fmt
 
 #endif
 
-#include "evas_gl_3d_common.h"
-
 #undef EAPI
 #define EAPI
 
