@@ -13,7 +13,7 @@ static char*
 _get_key_val(void *key)
 {
    static char buf[30];
-   snprintf(buf, sizeof(buf), "%ld", (size_t) key);
+   snprintf(buf, sizeof(buf), "%zu", (size_t) key);
    return buf;
 }
 
@@ -22,8 +22,13 @@ _construct_drawable_nodes(Efl_Canvas_Vg_Container *parent, const LOTLayerNode *l
 {
    if (!parent) return;
 
+   //This list is used for layer order verification
+   Eina_List *list = (Eina_List*) efl_canvas_vg_container_children_direct_get(parent);
+
    for (unsigned int i = 0; i < layer->mNodeList.size; i++)
      {
+        if (i > 0) list = eina_list_next(list);
+
         LOTNode *node = layer->mNodeList.ptr[i];
         if (!node) continue;
 
@@ -37,8 +42,10 @@ _construct_drawable_nodes(Efl_Canvas_Vg_Container *parent, const LOTLayerNode *l
                   image = efl_add(EFL_CANVAS_VG_IMAGE_CLASS, parent);
                   efl_key_data_set(parent, key, image);
                }
-             efl_gfx_entity_visible_set(image, EINA_TRUE);
-
+#if DEBUG
+        for (int i = 0; i < depth; i++) printf("    ");
+        printf("%s (%p)\n", efl_class_name_get(efl_class_get(image)), image);
+#endif
              Eina_Matrix3 m;
              eina_matrix3_identity(&m);
              eina_matrix3_values_set( &m,
@@ -46,10 +53,9 @@ _construct_drawable_nodes(Efl_Canvas_Vg_Container *parent, const LOTLayerNode *l
                                       node->mImageInfo.mMatrix.m21,  node->mImageInfo.mMatrix.m22, node->mImageInfo.mMatrix.m23,
                                       node->mImageInfo.mMatrix.m31,  node->mImageInfo.mMatrix.m32, node->mImageInfo.mMatrix.m33);
              efl_canvas_vg_node_transformation_set(image, &m);
-
              efl_canvas_vg_image_data_set(image, node->mImageInfo.data, EINA_SIZE2D(node->mImageInfo.width, node->mImageInfo.height));
-
              efl_gfx_color_set(image, node->mImageInfo.mAlpha, node->mImageInfo.mAlpha, node->mImageInfo.mAlpha, node->mImageInfo.mAlpha);
+             efl_gfx_entity_visible_set(image, EINA_TRUE);
 
              continue;
           }
@@ -62,7 +68,13 @@ _construct_drawable_nodes(Efl_Canvas_Vg_Container *parent, const LOTLayerNode *l
              efl_key_data_set(parent, key, shape);
           }
         else
-          efl_gfx_path_reset(shape);
+          {
+             efl_gfx_path_reset(shape);
+
+             //Layer order is mismatched!
+             if (eina_list_data_get(list) != shape)
+               efl_gfx_stack_raise_to_top(shape);
+          }
 
         //Skip Invisible Stroke?
         if (node->mStroke.enable && node->mStroke.width == 0)
@@ -74,10 +86,11 @@ _construct_drawable_nodes(Efl_Canvas_Vg_Container *parent, const LOTLayerNode *l
         const float *data = node->mPath.ptPtr;
         if (!data) continue;
 
+        if (node->keypath) efl_key_data_set(shape, "_lot_node_name", node->keypath);
         efl_gfx_entity_visible_set(shape, EINA_TRUE);
 #if DEBUG
         for (int i = 0; i < depth; i++) printf("    ");
-        printf("%s (%p)\n", efl_class_name_get(efl_class_get(shape)), shape);
+        printf("%s (%p) keypath : %s\n", efl_class_name_get(efl_class_get(shape)), shape, node->keypath);
 #endif
         //0: Path
         efl_gfx_path_reserve(shape, node->mPath.elmCount, node->mPath.ptCount);
@@ -184,6 +197,7 @@ _construct_drawable_nodes(Efl_Canvas_Vg_Container *parent, const LOTLayerNode *l
                           grad = efl_add(EFL_CANVAS_VG_GRADIENT_LINEAR_CLASS, parent);
                           efl_key_data_set(shape, key, grad);
                        }
+                     efl_gfx_entity_visible_set(grad, EINA_TRUE);
                      efl_gfx_gradient_linear_start_set(grad, node->mGradient.start.x, node->mGradient.start.y);
                      efl_gfx_gradient_linear_end_set(grad, node->mGradient.end.x, node->mGradient.end.y);
                   }
@@ -196,6 +210,7 @@ _construct_drawable_nodes(Efl_Canvas_Vg_Container *parent, const LOTLayerNode *l
                           grad = efl_add(EFL_CANVAS_VG_GRADIENT_RADIAL_CLASS, parent);
                           efl_key_data_set(shape, key, grad);
                        }
+                     efl_gfx_entity_visible_set(grad, EINA_TRUE);
                      efl_gfx_gradient_radial_center_set(grad, node->mGradient.center.x, node->mGradient.center.y);
                      efl_gfx_gradient_radial_focal_set(grad, node->mGradient.focal.x, node->mGradient.focal.y);
                      efl_gfx_gradient_radial_radius_set(grad, node->mGradient.cradius);
@@ -256,8 +271,12 @@ _construct_mask_nodes(Efl_Canvas_Vg_Container *parent, LOTMask *mask, int depth
    else
      efl_gfx_path_reset(shape);
 
-   efl_gfx_entity_visible_set(shape, EINA_TRUE);
+#if DEBUG
+        for (int i = 0; i < depth; i++) printf("    ");
+        printf("%s (%p)\n", efl_class_name_get(efl_class_get(shape)), shape);
+#endif
 
+   efl_gfx_entity_visible_set(shape, EINA_TRUE);
    efl_gfx_path_reserve(shape, mask->mPath.elmCount, mask->mPath.ptCount);
 
    for (size_t i = 0; i < mask->mPath.elmCount; i++)
@@ -293,7 +312,7 @@ _construct_mask_nodes(Efl_Canvas_Vg_Container *parent, LOTMask *mask, int depth
    efl_gfx_color_set(shape, r, g, b, a);
 }
 
-static void
+static Efl_Canvas_Vg_Container*
 _construct_masks(Efl_Canvas_Vg_Container *mtarget, LOTMask *masks, unsigned int mask_cnt, int depth)
 {
    char *key = NULL;
@@ -307,6 +326,13 @@ _construct_masks(Efl_Canvas_Vg_Container *mtarget, LOTMask *masks, unsigned int
         msource = efl_add(EFL_CANVAS_VG_CONTAINER_CLASS, mtarget);
         efl_key_data_set(mtarget, key, msource);
      }
+   efl_gfx_entity_visible_set(msource, EINA_TRUE);
+
+#if DEBUG
+        for (int i = 0; i < depth; i++) printf("    ");
+        printf("%s (%p), base mask => %p\n", efl_class_name_get(efl_class_get(msource)), msource, mtarget);
+        depth++;
+#endif
 
    //FIXME : EFL_GFX_VG_COMPOSITE_METHOD_ALPHA option is temporary
    //Currently matte alpha implements is same the mask intersect implement.
@@ -327,6 +353,13 @@ _construct_masks(Efl_Canvas_Vg_Container *mtarget, LOTMask *masks, unsigned int
              msource = efl_add(EFL_CANVAS_VG_CONTAINER_CLASS, mtarget);
              efl_key_data_set(mtarget, key, msource);
           }
+        efl_gfx_entity_visible_set(msource, EINA_TRUE);
+
+#if DEBUG
+        for (int i = 0; i < depth; i++) printf("    ");
+        printf("%s (%p), real mask:%d => %p\n", efl_class_name_get(efl_class_get(msource)), msource, mask->mMode, mtarget);
+#endif
+
         _construct_mask_nodes(msource, mask, depth + 1);
 
         Efl_Gfx_Vg_Composite_Method mask_mode;
@@ -349,16 +382,29 @@ _construct_masks(Efl_Canvas_Vg_Container *mtarget, LOTMask *masks, unsigned int
         efl_canvas_vg_node_comp_method_set(mtarget, msource, mask_mode);
         mtarget = msource;
      }
+   return mtarget;
+}
+
+static void
+_reset_vg_tree(Efl_VG *node)
+{
+   //Hide all nodes visibility
+   if (efl_isa(node, EFL_CANVAS_VG_CONTAINER_CLASS))
+     {
+        Eina_List *list = (Eina_List*) efl_canvas_vg_container_children_direct_get(node);
+        Eina_List *list2;
+        Efl_VG* child;
+        EINA_LIST_FOREACH(list, list2, child)
+          _reset_vg_tree(child);
+     }
+   efl_gfx_entity_visible_set(node, EINA_FALSE);
 }
 
 static void
 _update_vg_tree(Efl_Canvas_Vg_Container *root, const LOTLayerNode *layer, int depth EINA_UNUSED)
 {
-   if (!layer->mVisible)
-     {
-        efl_gfx_entity_visible_set(root, EINA_FALSE);
-        return;
-     }
+   if (!layer->mVisible) return;
+
    efl_gfx_entity_visible_set(root, EINA_TRUE);
    efl_gfx_color_set(root, layer->mAlpha, layer->mAlpha, layer->mAlpha, layer->mAlpha);
 
@@ -388,7 +434,7 @@ _update_vg_tree(Efl_Canvas_Vg_Container *root, const LOTLayerNode *layer, int de
           }
 #if DEBUG
         for (int i = 0; i < depth; i++) printf("    ");
-        printf("%s (%p) matte:%d => %p\n", efl_class_name_get(efl_class_get(ctree)), ctree, matte_mode, ptree);
+        printf("%s (%p) matte:%d => %p %s\n", efl_class_name_get(efl_class_get(ctree)), ctree, matte_mode, ptree, clayer->keypath);
 #endif
         _update_vg_tree(ctree, clayer, depth+1);
 
@@ -404,6 +450,8 @@ _update_vg_tree(Efl_Canvas_Vg_Container *root, const LOTLayerNode *layer, int de
              mlayer = clayer;
              if (!mtarget) mtarget = ctree;
           }
+        else
+           mtarget = NULL;
 
         ptree = ctree;
 
@@ -431,18 +479,108 @@ _update_vg_tree(Efl_Canvas_Vg_Container *root, const LOTLayerNode *layer, int de
               matte_mode = 0;
               break;
           }
+
+        //Construct node that have mask.
+        if (mlayer && mtarget)
+          ptree = _construct_masks(mtarget, mlayer->mMaskList.ptr, mlayer->mMaskList.size, depth + 1);
      }
 
    //Construct drawable nodes.
    if (layer->mNodeList.size > 0)
      _construct_drawable_nodes(root, layer, depth);
 
-   //Construct node that have mask.
-   if (mlayer)
-     _construct_masks(mtarget, mlayer->mMaskList.ptr, mlayer->mMaskList.size, depth);
 }
 #endif
 
+#ifdef BUILD_VG_LOADER_JSON
+void
+_value_provider_override(Vg_File_Data *vfd)
+{
+   Lottie_Animation *lot_anim = (Lottie_Animation *) vfd->loader_data;
+
+   Eina_List *l;
+   Efl_Gfx_Vg_Value_Provider *vp;
+   EINA_LIST_FOREACH(vfd->vp_list, l, vp)
+     {
+        const char *keypath;
+        Efl_Gfx_Vg_Value_Provider_Change_Flag flag;
+        flag = efl_gfx_vg_value_provider_changed_flag_get(vp);
+
+        if (flag & EFL_GFX_VG_VALUE_PROVIDER_CHANGE_FLAG_FILL_COLOR)
+          {
+             int r, g, b, a;
+             r = g = b = a = 0;
+             efl_gfx_vg_value_provider_fill_color_get(vp, &r, &g, &b, &a);
+             keypath = efl_gfx_vg_value_provider_keypath_get(vp);
+
+             lottie_animation_property_override(lot_anim, LOTTIE_ANIMATION_PROPERTY_FILLCOLOR, (char*)keypath, r / 255.0, g / 255.0, b / 255.0);
+             lottie_animation_property_override(lot_anim, LOTTIE_ANIMATION_PROPERTY_FILLOPACITY, (char*)keypath, (a / 255.0) * 100.0);
+          }
+        if (flag & EFL_GFX_VG_VALUE_PROVIDER_CHANGE_FLAG_STROKE_COLOR)
+          {
+             int r, g, b, a;
+             r = g = b = a = 0;
+             efl_gfx_vg_value_provider_stroke_color_get(vp, &r, &g, &b, &a);
+             keypath = efl_gfx_vg_value_provider_keypath_get(vp);
+
+             lottie_animation_property_override(lot_anim, LOTTIE_ANIMATION_PROPERTY_STROKECOLOR, (char*)keypath, r / 255.0, g / 255.0, b / 255.0);
+             lottie_animation_property_override(lot_anim, LOTTIE_ANIMATION_PROPERTY_STROKEOPACITY, (char*)keypath, (a / 255.0) * 100.0);
+          }
+        if (flag & EFL_GFX_VG_VALUE_PROVIDER_CHANGE_FLAG_STROKE_WIDTH)
+          {
+             double w;
+             w = efl_gfx_vg_value_provider_stroke_width_get(vp);
+             keypath = efl_gfx_vg_value_provider_keypath_get(vp);
+
+             lottie_animation_property_override(lot_anim, LOTTIE_ANIMATION_PROPERTY_STROKEWIDTH, (char*)keypath, w);
+          }
+        if (flag & EFL_GFX_VG_VALUE_PROVIDER_CHANGE_FLAG_TRANSFORM_MATRIX)
+          {
+             Eina_Matrix4 m, *orig_m;
+             double tx, ty, sx, sy, radian_z, si, cs;
+
+             orig_m = efl_gfx_vg_value_provider_transform_get(vp);
+             if (!orig_m) continue;
+
+             /*
+              * NOTE: We need to impelements 3-axis transform.
+              * now lottie animation provide z projection transform.
+              * In this cace, we calcuate to T * R * S order.
+              */
+             eina_matrix4_copy(&m, orig_m);
+             keypath = efl_gfx_vg_value_provider_keypath_get(vp);
+
+             // Calc Translate
+             eina_matrix4_values_get(&m, NULL, NULL, NULL, &tx,
+                                         NULL, NULL, NULL, &ty,
+                                         NULL, NULL, NULL, NULL,
+                                         NULL, NULL, NULL, NULL);
+             eina_matrix4_translate(&m, -1 * tx, -1 * ty, 0);
+
+             // Calc Rotate
+             eina_matrix4_values_get(&m,  &cs, NULL, NULL, NULL,
+                                          &si, NULL, NULL, NULL,
+                                         NULL, NULL, NULL, NULL,
+                                         NULL, NULL, NULL, NULL);
+             radian_z = atan2(si, cs);
+             eina_matrix4_rotate(&m, -1 * radian_z, EINA_MATRIX_AXIS_Z);
+
+             // Calc Scale
+             eina_matrix4_values_get(&m,  &sx, NULL, NULL, NULL,
+                                         NULL,  &sy, NULL, NULL,
+                                         NULL, NULL, NULL, NULL,
+                                         NULL, NULL, NULL, NULL);
+
+             lottie_animation_property_override(lot_anim, LOTTIE_ANIMATION_PROPERTY_TR_SCALE, (char*)keypath, 100.0 * sx, 100.0 * sy);
+             lottie_animation_property_override(lot_anim, LOTTIE_ANIMATION_PROPERTY_TR_ROTATION, (char*)keypath, radian_z * (180.0 / M_PI));
+             lottie_animation_property_override(lot_anim, LOTTIE_ANIMATION_PROPERTY_TR_POSITION, (char*)keypath, tx, ty);
+          }
+     }
+}
+
+#endif
+
+
 Eina_Bool
 vg_common_json_create_vg_node(Vg_File_Data *vfd)
 {
@@ -450,14 +588,12 @@ vg_common_json_create_vg_node(Vg_File_Data *vfd)
    Lottie_Animation *lot_anim = (Lottie_Animation *) vfd->loader_data;
    if (!lot_anim) return EINA_FALSE;
 
+   if (vfd->vp_list) _value_provider_override(vfd);
+
    unsigned int frame_num = (vfd->anim_data) ? vfd->anim_data->frame_num : 0;
    const LOTLayerNode *tree =
       lottie_animation_render_tree(lot_anim, frame_num,
                                    vfd->view_box.w, vfd->view_box.h);
-#if DEBUG
-   printf("%s (%p)\n", efl_class_name_get(efl_class_get(vfd->root)), vfd->root);
-#endif
-
    //Root node
    Efl_Canvas_Vg_Container *root = vfd->root;
    if (!root)
@@ -468,6 +604,12 @@ vg_common_json_create_vg_node(Vg_File_Data *vfd)
         if (tree->keypath) efl_key_data_set(root, "_lot_node_name", tree->keypath);
         vfd->root = root;
      }
+   else _reset_vg_tree(root);
+
+#if DEBUG
+   printf("%s (%p)\n", efl_class_name_get(efl_class_get(vfd->root)), vfd->root);
+#endif
+
    _update_vg_tree(root, tree, 1);
 #else
    return EINA_FALSE;
