@@ -368,7 +368,7 @@ EFL_START_TEST(eolian_typedef)
    fail_if(strcmp(type_name, "List_Objects"));
    fail_if(!(type = eolian_typedecl_base_type_get(tdl)));
    fail_if(!(type_name = eolian_type_c_type_get(type)));
-   fail_if(strcmp(type_name, "Eina_List *"));
+   fail_if(strcmp(type_name, "Eina_Iterator *"));
    eina_stringshare_del(type_name);
    fail_if(!(type = eolian_type_base_type_get(type)));
    fail_if(!!eolian_type_next_type_get(type));
@@ -639,7 +639,7 @@ EFL_START_TEST(eolian_simple_parsing)
    fail_if(!expr);
    v = eolian_expression_eval(expr, EOLIAN_MASK_FLOAT);
    fail_if(v.type != EOLIAN_EXPR_DOUBLE);
-   fail_if(v.value.d != 1337.6);
+   fail_if(!EINA_DBL_EQ(v.value.d, 1337.6));
    fail_if(!(eina_iterator_next(iter, (void**)&param)));
    fail_if(eolian_parameter_direction_get(param) != EOLIAN_PARAMETER_IN);
    fail_if(strcmp(eolian_type_short_name_get(eolian_parameter_type_get(param)), "int"));
@@ -805,6 +805,7 @@ EFL_START_TEST(eolian_error)
    const Eolian_Function *f1, *f2;
    const Eolian_Type *rtp1, *rtp2;
    const Eolian_Error *err1, *err2;
+   Eina_Stringshare *str;
 
    Eolian_State *eos = eolian_state_new();
 
@@ -822,6 +823,9 @@ EFL_START_TEST(eolian_error)
    fail_if(eolian_type_type_get(rtp1) != EOLIAN_TYPE_ERROR);
    fail_if(eolian_type_next_type_get(rtp1) != NULL);
    fail_if(strcmp(eolian_type_name_get(rtp1), "Foo"));
+   fail_if(!(str = eolian_type_c_type_get(rtp1)));
+   fail_if(strcmp(str, "Eina_Error"));
+   eina_stringshare_del(str);
    fail_if(!(err1 = eolian_type_error_get(rtp1)));
    fail_if(strcmp(eolian_error_message_get(err1), "something bad happened"));
 
@@ -1150,7 +1154,8 @@ EFL_START_TEST(eolian_docs)
    fail_if(strcmp(eolian_documentation_summary_get(doc),
                   "Docs for enum Bar."));
    fail_if(eolian_documentation_description_get(doc));
-   fail_if(eolian_documentation_since_get(doc));
+   fail_if(strcmp(eolian_documentation_since_get(doc),
+                  "1.55"));
 
    fail_if(!(efl = eolian_typedecl_enum_field_get(tdl, "blah")));
    fail_if(eolian_typedecl_enum_field_documentation_get(efl));
@@ -1174,7 +1179,7 @@ EFL_START_TEST(eolian_docs)
    fail_if(strcmp(eolian_documentation_description_get(doc),
                   "More docs for typedef. See @Bar."));
    fail_if(strcmp(eolian_documentation_since_get(doc),
-                  "2.0"));
+                  "1.1337"));
 
    fail_if(!(var = eolian_unit_constant_by_name_get(unit, "pants")));
    fail_if(!(doc = eolian_constant_documentation_get(var)));
@@ -1236,7 +1241,7 @@ EFL_START_TEST(eolian_docs)
                   "Property common documentation."));
    fail_if(eolian_documentation_description_get(doc));
    fail_if(strcmp(eolian_documentation_since_get(doc),
-                  "1.18"));
+                  "1.66"));
    fail_if(!(doc = eolian_implement_documentation_get(fimp, EOLIAN_PROP_GET)));
    fail_if(strcmp(eolian_documentation_summary_get(doc),
                   "Get documentation."));
@@ -1263,6 +1268,15 @@ EFL_START_TEST(eolian_docs)
                   "Event docs."));
    fail_if(eolian_documentation_description_get(doc));
 
+   fail_if(!(fid = eolian_class_function_by_name_get(class, "no_doc_meth", EOLIAN_METHOD)));
+   fail_if(!(fimp = eolian_function_implement_get(fid)));
+   fail_if((doc = eolian_implement_documentation_get(fimp, EOLIAN_METHOD)));
+
+   fail_if(!(fid = eolian_class_function_by_name_get(class, "doc_with_empty_doc", EOLIAN_METHOD)));
+   fail_if(!(fimp = eolian_function_implement_get(fid)));
+   fail_if(!(doc = eolian_implement_documentation_get(fimp, EOLIAN_METHOD)));
+   fail_if(eolian_documentation_summary_get(doc));
+
    eolian_state_free(eos);
 }
 EFL_END_TEST
