@@ -9,6 +9,7 @@
 
 #include <Evas.h>
 #include <Ecore_Evas.h>
+#include <Ecore.h>
 
 #include "evas_suite.h"
 #include "evas_tests_helpers.h"
@@ -1117,6 +1118,92 @@ EFL_START_TEST(evas_object_image_9patch)
 }
 EFL_END_TEST
 
+EFL_START_TEST(evas_object_image_save_from_proxy)
+{
+   Evas *e;
+   Evas_Object *obj, *proxy, *ref;
+   int w, h, r_w, r_h;
+   const uint32_t *d, *r_d;
+   const char *img_path, *img_path2;
+   Eina_Bool ret;
+
+   e = _setup_evas();
+   img_path = TESTS_IMG_DIR "/Pic1.png";
+   img_path2 = TESTS_IMG_DIR "/Pic1_saved.png";
+
+   obj = evas_object_image_add(e);
+   proxy = evas_object_image_add(e);
+   ref = evas_object_image_add(e);
+
+   evas_object_image_file_set(obj, img_path, NULL);
+   fail_if(evas_object_image_load_error_get(obj) != EVAS_LOAD_ERROR_NONE);
+   evas_object_image_size_get(obj, &w, &h);
+   d = evas_object_image_data_get(obj, EINA_FALSE);
+
+   evas_object_image_source_set(proxy, obj);
+   ret = evas_object_image_save(proxy, img_path2, NULL, NULL);
+   fail_if(!ret);
+
+   evas_object_image_file_set(ref, img_path2, NULL);
+   fail_if(evas_object_image_load_error_get(ref) != EVAS_LOAD_ERROR_NONE);
+   evas_object_image_size_get(ref, &r_w, &r_h);
+   r_d = evas_object_image_data_get(ref, EINA_FALSE);
+
+   fail_if(w != r_w || h != r_h);
+   fail_if(memcmp(d, r_d, w * h * 4));
+
+   evas_object_del(proxy);
+   evas_object_del(obj);
+   evas_object_del(ref);
+
+   remove(img_path2);
+
+   evas_free(e);
+}
+EFL_END_TEST
+
+static void
+_preload(void *data, Evas *e EINA_UNUSED, Evas_Object *obj, void *event_info EINA_UNUSED)
+{
+   int *called = data;
+   int w, h;
+
+   *called = 1;
+   evas_object_image_size_get(obj, &w, &h);
+   ck_assert_int_ge(w, 1);
+   ck_assert_int_ge(h, 1);
+   ecore_main_loop_quit();
+}
+
+EFL_START_TEST(evas_object_image_load_head_skip)
+{
+   Evas *e;
+   Evas_Object *obj;
+   const char *img_path;
+   int called = 0;
+
+   e = _setup_evas();
+   img_path = TESTS_IMG_DIR "/Pic1.png";
+
+   obj = evas_object_image_filled_add(e);
+   evas_object_resize(obj, 100, 100);
+
+   evas_object_image_load_head_skip_set(obj, EINA_TRUE);
+   evas_object_event_callback_add(obj, EVAS_CALLBACK_IMAGE_PRELOADED, _preload, &called);
+   evas_object_image_file_set(obj, img_path, NULL);
+   evas_object_image_preload(obj, EINA_FALSE);
+
+   ck_assert(!efl_file_mmap_get(obj));
+   ck_assert(!efl_file_loaded_get(obj));
+   ecore_main_loop_begin();
+   ck_assert_int_eq(called, 1);
+   ck_assert(efl_file_loaded_get(obj));
+   ck_assert(efl_file_mmap_get(obj));
+
+   evas_free(e);
+}
+EFL_END_TEST
+
 void evas_test_image_object(TCase *tc)
 {
    tcase_add_test(tc, evas_object_image_api);
@@ -1143,6 +1230,8 @@ void evas_test_image_object(TCase *tc)
    tcase_add_test(tc, evas_object_image_partially_load_orientation);
    tcase_add_test(tc, evas_object_image_cached_data_comparision);
    tcase_add_test(tc, evas_object_image_9patch);
+   tcase_add_test(tc, evas_object_image_save_from_proxy);
+   tcase_add_test(tc, evas_object_image_load_head_skip);
 }
 
 
